import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.*;
import java.awt.Toolkit;
import javax.swing.*;


/**
 * This is the Database Monitor Application (DBA GUI). It contains all the components
 * needed for the graphical user interface and interacts wth DBAModel.java to implement
 * all the features.
 * This class should be invoked by the DBALoginDialog.java class
 *
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
public class DBAView implements Observer
{

    Toolkit kit = Toolkit.getDefaultToolkit();
    Dimension screenSize = kit.getScreenSize();
    int screenHt = screenSize.height;
    int screenWt = screenSize.width;

    //Set to true for debugging
    private boolean debug = false;

    private DBAModel model = new DBAModel();
    private JFrame frame = new JFrame("Database Monitor Application");
    private JTabbedPane pane = new JTabbedPane();

    private JPanel boxPanel = new JPanel();
    private JPanel topPanel = new JPanel();
    private JPanel notPanel = new JPanel();
    private JPanel validPanel = new JPanel();
    private JPanel datePanel = new JPanel();
    private JPanel savePanel = new JPanel();
    private JPanel problemsPanel = new JPanel();
    private JPanel NotorizationIntervalPanel = new JPanel();
    private JPanel NotorizationTimePanel = new JPanel();  
    private JPanel ValidationIntervalPanel = new JPanel();  
    private JPanel ValidationTimePanel = new JPanel();
    private JPanel CostPerUnitPanel = new JPanel();
    private JPanel CostPanel = new JPanel();
    private JPanel costPanel1 = new JPanel(); 
    private JPanel costPanel2 = new JPanel(); 

    //Top Panel Components
    private JComboBox databaseDropDown = new JComboBox();
    private JLabel csiLabel = new JLabel("CSI:");

    //Configuration Stuff (notPanel/validPanel components)
    String[] algs = {"Monochromatic", "RGBY", "Tiled Bitmap", "A3D"};
    private JComboBox algorithmDropDown = new JComboBox(algs);
    JSlider slider = new JSlider(JSlider.VERTICAL,0, 30, 15);
    Integer[] days = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31};
    private JComboBox notzDaysDropDown = new JComboBox(days);
    //private JComboBox notzDaysDropDown1 = new JComboBox(days);
    //private JComboBox notzDaysDropDown2 = new JComboBox(days);
    Integer[] hours = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};
    private JComboBox notzHoursDropDown = new JComboBox(hours);
    private JComboBox notzHoursDropDown1 = new JComboBox(hours);
    private JComboBox notzHoursDropDown2 = new JComboBox(hours);
    Integer[] mins = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59};
String[] mins1 = {"00","01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","43","44","45","46","47","48","49","50","51","52","53","54","55","56","57","58","59"};
    private JComboBox notzMinutesDropDown = new JComboBox(mins);
    private JComboBox notzMinutesDropDown1 = new JComboBox(mins);
    private JComboBox notzMinutesDropDown2 = new JComboBox(mins);
    Integer[] Ns = {1,2,4,8,16,32,64,128};
    private JComboBox NDropDown = new JComboBox(Ns);
    Integer[] VNs = {1,2,3,4,5,6,7,8,9,10};
    private JComboBox VNDropDown = new JComboBox(VNs);
    Integer[] CorruptionNum = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59};
    private JComboBox CorruptionNumDropDown = new JComboBox(CorruptionNum);
    /* Michael Patterson 10/12/08 - Changed the scope on these fields to apply to the entire class */
    private JTextField notDaysTextField = new JTextField("0");
    private JTextField notHoursTextField = new JTextField("0");
    private JTextField notMinsTextField = new JTextField("1");
    private JTextField notDaysTextField1 = new JTextField("0");
    private JTextField notHoursTextField1 = new JTextField("0");
    private JTextField notMinsTextField1 = new JTextField("1");
    private JTextField costPerUnitField = new JTextField("0.01");
    private JLabel costText1 = new JLabel();
    private JLabel costText2 = new JLabel();
    private JLabel singleFA = new JLabel();
    private JLabel multiFA = new JLabel();
    /* MP - End field scope changes */

    private JSlider valSlider = new JSlider(JSlider.HORIZONTAL,1, 10, 5);

    private JSeparator separator1 = new JSeparator();
    private JSeparator separator2 = new JSeparator();

    //Time Stuff (datePanel components)
    Integer[] years = {2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020};
    String [] months1 = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
    Integer[] months = {1,2,3,4,5,6,7,8,9,10,11,12};

    //String [] Time_hours = {"12am","1am","2am","3am","4am","5am","6am","7am","8am","9am","10am","11am","12pm","1pm","2pm","3pm","4pm","5pm","6pm","7pm","8pm","9pm","10pm","11pm"};



    private JComboBox year = new JComboBox(years);
    private JComboBox month = new JComboBox(months);
    private JComboBox day = new JComboBox(days);
    //private JComboBox hour = new JComboBox(Time_hours);
    private JComboBox hour = new JComboBox(hours);


    private JComboBox minute = new JComboBox(mins);

    //Tamper Detection Components (problemsPanel components)
    private DefaultListModel activitiesListModel = new DefaultListModel();
    private JList activitiesList = new JList(activitiesListModel);
    private JScrollPane scrollingActivitiesList;
    private JButton saveButton = new JButton("Save These Settings");
   private JPanel PtopPanel = new JPanel();
    private JPanel PtopPanel1 = new JPanel();
    private JPanel PtopPanel2 = new JPanel();
    private JButton viewBtn = new JButton("View Database Corruption Graphically");
    private JTextField FAcostText1 = new JTextField();
    private JTextField FAcostText2 = new JTextField();    
    private JTextField FAcostText3 = new JTextField();
    private JPanel FAcostPanel1 = new JPanel();
    private JPanel FAcostPanel2 = new JPanel();
    private JPanel FAcostPanel3 = new JPanel();

    //Menu items
    private JMenuBar menuBar = new JMenuBar();
    private JMenu menu = new JMenu("File");
    private JMenuItem exitItem = new JMenuItem("Exit");
    
    //Michael Patterson 10/12/08 - Display values in Units or $
    private Boolean bUseUnits = false;	//Always default to $

    /**
     * Sets up and displays the application with the given DBA as the currently logged in employee.
     *
     * @param dbaEmail the email of the currently logged in DBA
     */
/* Modified by Huilong on 5/14/2007 */
    /* Michael Patterson 10/8/2008 - Adding boolean for Unit display */    
    //public DBAView(String name, String dbaEmail)
    public DBAView(String name, String dbaEmail, Boolean bSetUnits)
    {
    	bUseUnits = bSetUnits;
    /* MP - End of Unit display boolean */    	
        frame.setTitle("DBA: "+name+" Email:"+dbaEmail);
/* End Huilong */
        DatabaseConfig overallConfig = model.getCurrentOverallConfig();

        if(dbaEmail != null)
        {
            model.setCurrentDBA(dbaEmail);
            if(debug)   { System.out.println("Set DBA to: " + dbaEmail);    }
        }

        model.addObserver(this);

        loadDatabases();

//      -----------Setup menu-------------
        menuBar.add(menu);
        exitItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                System.exit(0);
            }
        });
        menu.add(exitItem);
        frame.setJMenuBar(menuBar);

        //Limit the algorithm list based on the minimum provided by the overall settings
        if(overallConfig.getAlgorithm().equals("RGBY"))
        {
            String[] algorithms = {"RGBY", "Tiled Bitmap", "A3D"};
            algorithmDropDown = new JComboBox(algorithms);
        }
        else if(overallConfig.getAlgorithm().equals("Tiled Bitmap"))
        {
            String[] algorithms = {"Tiled Bitmap", "A3D"};
            algorithmDropDown = new JComboBox(algorithms);
        }
        else if(overallConfig.getAlgorithm().equals("A3D"))
        {
            String[] algorithms = {"A3D"};
            algorithmDropDown = new JComboBox(algorithms);
        }

        //Limit the validation slider based on the minimum provided by the overall settings
         //For now We don't need that
        if(overallConfig.getValidationFactor() != 0)
        {
            valSlider = new JSlider(JSlider.HORIZONTAL, 1, overallConfig.getValidationFactor(), 1);
        } 
          

        //Limit the notarization choices based on the minimum provided by the overall settings
        for(int i = 0; i <= overallConfig.getNotzIntervalDays(); i++)
        {
            notzDaysDropDown.addItem(i);
        }

        databaseDropDown.setAlignmentX(Component.CENTER_ALIGNMENT);
        databaseDropDown.setMaximumSize(new Dimension(200,30));
        databaseDropDown.setMinimumSize(new Dimension(200,30));
        topPanel.add(databaseDropDown);
        topPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        csiLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        topPanel.add(csiLabel);

        databaseDropDown.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae)
            {
                if(databaseDropDown.getSelectedItem() != null)
                {
                    try
                    {
                        String selectedText = databaseDropDown.getSelectedItem().toString();
                        StringTokenizer tokenizer = new StringTokenizer(selectedText, " ()");

                        model.setSelectedDB(tokenizer.nextToken(), tokenizer.nextToken());
                        update(null, null);
                    }
                    catch(Exception e)
                    {}
                }

            }
        });

        slider.setMajorTickSpacing(10);
        slider.setMinorTickSpacing(1);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.setSnapToTicks(true);
        
        /* Michael Patterson 10/12/08 - Adding listener for Predicted Cost to recalculate when any of the
         * setting fields are changed.
         */
        ActionListener aCalcFC = new ActionListener()
        {
        	public void actionPerformed(ActionEvent ae)
        	{
        		calculateFC();
        	}
        };
        /* MP - End aCalcFC ActionListener. */

        notPanel.setLayout(new BoxLayout(notPanel, BoxLayout.X_AXIS));
        JLabel notarizationLabel = new JLabel("Detection Resolution Unit:");
        notarizationLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        notPanel.add(notarizationLabel);
        notPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        notzDaysDropDown.setSelectedItem("0");
        notzDaysDropDown.setMaximumSize(new Dimension(50,45));
        notzDaysDropDown.setMinimumSize(new Dimension(50,45));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notzDaysDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        notPanel.add(notzDaysDropDown);
        notPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel dayLabel = new JLabel("days");
        dayLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        notPanel.add(dayLabel);
        notPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        notzHoursDropDown.setMaximumSize(new Dimension(50,45));
        notzHoursDropDown.setMinimumSize(new Dimension(50,45));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notzHoursDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        notPanel.add(notzHoursDropDown);
        notPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel hourLabel = new JLabel("hrs");
        hourLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        notPanel.add(hourLabel);
        notPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        notzMinutesDropDown.setMaximumSize(new Dimension(50,45));
        notzMinutesDropDown.setMinimumSize(new Dimension(50,45));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notzMinutesDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        notPanel.add(notzMinutesDropDown);
        notPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel minLabel = new JLabel("mins.");
        minLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        notPanel.add(minLabel);
        notPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
   
        valSlider.setMajorTickSpacing(1);
        valSlider.setPaintTicks(true);
        valSlider.setPaintLabels(true);
        valSlider.setSnapToTicks(true);
        validPanel.setLayout(new BoxLayout(validPanel, BoxLayout.X_AXIS));
        //JLabel validLabel = new JLabel("Validation Interval:");
        //validLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        //validPanel.add(validLabel);
        //validPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //validPanel.add(valSlider);
        validPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel algorithmLabel = new JLabel("Forensic Algorithm:");
        algorithmLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        validPanel.add(algorithmLabel);
        algorithmDropDown.setMaximumSize(new Dimension(150,30));
        algorithmDropDown.setMinimumSize(new Dimension(150,30));
        validPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        algorithmDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */        
        validPanel.add(algorithmDropDown);

        NotorizationIntervalPanel.setLayout(new BoxLayout(NotorizationIntervalPanel,BoxLayout.X_AXIS));
	JLabel NLabel = new JLabel("Number of Resolution Units Between Notarizations ");
        NotorizationIntervalPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
        NotorizationIntervalPanel.add(NLabel);
        NotorizationIntervalPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        NDropDown.setMaximumSize(new Dimension(60,30));
        NDropDown.setMinimumSize(new Dimension(60,30));
        NotorizationIntervalPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        NDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        NotorizationIntervalPanel.add(NDropDown);


        NotorizationTimePanel.setLayout(new BoxLayout(NotorizationTimePanel,BoxLayout.X_AXIS));
	JLabel NotorizationIntervalLabel = new JLabel("Time between notarizations:");
        NotorizationTimePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
        NotorizationTimePanel.add(NotorizationIntervalLabel);
        NotorizationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //notzDaysDropDown1.setMaximumSize(new Dimension(50,45));
        //notzDaysDropDown1.setMinimumSize(new Dimension(50,45));
        //NotorizationTimePanel.add(notzDaysDropDown1);
        /* Michael Patterson 10/12/08 - Changed notDaysTextField scope to entire class */
        //JTextField notDaysTextField = new JTextField("0");
        notDaysTextField.setMaximumSize(new Dimension(40,40));
        notDaysTextField.setMinimumSize(new Dimension(40,40));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notDaysTextField.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        NotorizationTimePanel.add(notDaysTextField);
        NotorizationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel dayLabel1 = new JLabel("days");
        dayLabel1.setAlignmentX(Component.CENTER_ALIGNMENT);
        NotorizationTimePanel.add(dayLabel1);
        NotorizationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //notzHoursDropDown1.setMaximumSize(new Dimension(50,45));
        //notzHoursDropDown1.setMinimumSize(new Dimension(50,45));
        //NotorizationTimePanel.add(notzHoursDropDown1);
        /* Michael Patterson 10/12/08 - Changed notHoursTextField scope to entire class */
        //JTextField notHoursTextField = new JTextField("0");
        notHoursTextField.setMaximumSize(new Dimension(40,40));
        notHoursTextField.setMinimumSize(new Dimension(40,40));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notHoursTextField.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        NotorizationTimePanel.add(notHoursTextField);
        NotorizationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel hourLabel1 = new JLabel("hrs");
        hourLabel1.setAlignmentX(Component.CENTER_ALIGNMENT);
        NotorizationTimePanel.add(hourLabel1);
        NotorizationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //notzMinutesDropDown1.setMaximumSize(new Dimension(50,45));
        //notzMinutesDropDown1.setMinimumSize(new Dimension(50,45));
        //NotorizationTimePanel.add(notzMinutesDropDown1);

        /* Michael Patterson 10/12/08 - Changed notMinsTextField scope to entire class */
        //JTextField notMinsTextField = new JTextField("1");
        notMinsTextField.setMaximumSize(new Dimension(40,40));
        notMinsTextField.setMinimumSize(new Dimension(40,40));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notMinsTextField.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        NotorizationTimePanel.add(notMinsTextField);
        NotorizationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel minLabel1 = new JLabel("mins.");
        minLabel1.setAlignmentX(Component.CENTER_ALIGNMENT);
        NotorizationTimePanel.add(minLabel1);
        NotorizationTimePanel.setAlignmentX(Component.CENTER_ALIGNMENT);



        ValidationIntervalPanel.setLayout(new BoxLayout( ValidationIntervalPanel,BoxLayout.X_AXIS));
	JLabel vLabel = new JLabel("Number of Notarizations Between Validations ");
        ValidationIntervalPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
        ValidationIntervalPanel.add(vLabel);
        ValidationIntervalPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        VNDropDown.setMaximumSize(new Dimension(60,30));
        VNDropDown.setMinimumSize(new Dimension(60,30));
        ValidationIntervalPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        VNDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        ValidationIntervalPanel.add(VNDropDown);


	ValidationTimePanel.setLayout(new BoxLayout( ValidationTimePanel,BoxLayout.X_AXIS));
	JLabel ValidationIntervalLabel = new JLabel("Time between validations:");
	ValidationTimePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
	ValidationTimePanel.add(ValidationIntervalLabel);
	ValidationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //notzDaysDropDown2.setMaximumSize(new Dimension(50,45));
        //notzDaysDropDown2.setMinimumSize(new Dimension(50,45));
	//ValidationTimePanel.add(notzDaysDropDown2);
    /* Michael Patterson 10/12/08 - Changed notDaysTextField1 scope to entire class */
        //JTextField notDaysTextField1 = new JTextField("0");
        notDaysTextField1.setMaximumSize(new Dimension(40,40));
        notDaysTextField1.setMinimumSize(new Dimension(40,40));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notDaysTextField1.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        ValidationTimePanel.add(notDaysTextField1);
	ValidationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel dayLabel2 = new JLabel("days");
        dayLabel2.setAlignmentX(Component.CENTER_ALIGNMENT);
	ValidationTimePanel.add(dayLabel2);
	ValidationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //notzHoursDropDown2.setMaximumSize(new Dimension(50,45));
        //notzHoursDropDown2.setMinimumSize(new Dimension(50,45));
	//ValidationTimePanel.add(notzHoursDropDown2);
    /* Michael Patterson 10/12/08 - Changed notHoursTextField1 scope to entire class */
		//JTextField notHoursTextField1 = new JTextField("0");
        notHoursTextField1 .setMaximumSize(new Dimension(40,40));
        notHoursTextField1 .setMinimumSize(new Dimension(40,40));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notHoursTextField1.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        ValidationTimePanel.add(notHoursTextField1 );
	ValidationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel hourLabel2 = new JLabel("hrs");
        hourLabel2.setAlignmentX(Component.CENTER_ALIGNMENT);
	ValidationTimePanel.add(hourLabel2);
 	ValidationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //notzMinutesDropDown2.setMaximumSize(new Dimension(50,45));
        //notzMinutesDropDown2.setMinimumSize(new Dimension(50,45));
	//ValidationTimePanel.add(notzMinutesDropDown2);
    /* Michael Patterson 10/12/08 - Changed notMinsTextField1 scope to entire class */ 	
        //JTextField notMinsTextField1 = new JTextField("1");
        notMinsTextField1.setMaximumSize(new Dimension(40,40));
        notMinsTextField1.setMinimumSize(new Dimension(40,40));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        notMinsTextField1.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        ValidationTimePanel.add(notMinsTextField1);
	ValidationTimePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel minLabel2 = new JLabel("mins.");
        minLabel2.setAlignmentX(Component.CENTER_ALIGNMENT);
	ValidationTimePanel.add(minLabel2);
  	ValidationTimePanel.setAlignmentX(Component.CENTER_ALIGNMENT);

  	/* Michael Patterson 10/12/08 - Adding panel and field for adjusting the cost per unit */
  	if(!bUseUnits)
  	{
  		CostPerUnitPanel.setLayout(new BoxLayout( CostPerUnitPanel,BoxLayout.X_AXIS));
  		JLabel CostPerUnitLabel = new JLabel("Cost Per Unit: $ ");
  		CostPerUnitPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
  		CostPerUnitPanel.add(CostPerUnitLabel);
        costPerUnitField.setMaximumSize(new Dimension(60,40));
        costPerUnitField.setMinimumSize(new Dimension(60,40));
        costPerUnitField.addActionListener(aCalcFC);
        CostPerUnitPanel.add(costPerUnitField);
        CostPerUnitPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
  	}	
	/* MP - End adding of panel for adjusting cost per unit */

        CostPanel.setLayout(new BoxLayout(CostPanel,BoxLayout.Y_AXIS));
        JLabel CostLabel1 = new JLabel("Predicted Cost:");
  	CostLabel1.setAlignmentX(Component.CENTER_ALIGNMENT);
 
        JLabel CostLabel2 = new JLabel("Tampering Detection:");
  	CostLabel2.setAlignmentX(Component.CENTER_ALIGNMENT);
        JLabel CostLabel3 = new JLabel("Forensic Analysis (Worst Case):");     
  	CostLabel3.setAlignmentX(Component.CENTER_ALIGNMENT);
        costPanel1.setLayout(new BoxLayout(costPanel1, BoxLayout.X_AXIS));
        costPanel1.add(Box.createRigidArea(new Dimension(20, 25)));
        /* Michael Patterson 10/08/2008 - Selecting to display in Units or $ */
		String sNotarizationPerDay;
		if(bUseUnits)
		{
			sNotarizationPerDay = "Per Day (in Units): ";
		}
		else
		{
			sNotarizationPerDay = "Per Day $: ";
		}
		//costPanel1.add(new JLabel("Per Day: $ "));
		costPanel1.add(new JLabel(sNotarizationPerDay));
        /* MP - End changes for display in Units or $ */
		/* Michael Patterson 10/08/2008 - Calculating the cost per day and year for notarizations */
		calculateFC();
        //JTextField costText1 = new JTextField("3.25");
		//JTextField costText1 = new JTextField(dcForm.format(fc.NP_mono(notzResolution, notzInterval/valInterval)));
	costText1.setMaximumSize(new Dimension(50,45));
	costText1.setMinimumSize(new Dimension(50,45));
        //JTextField costText2 = new JTextField("1186.25");
	//JTextField costText2 = new JTextField(dcForm.format((fc.NP_mono(notzResolution, notzInterval/valInterval) * 365)));
        /* MP - End changes for calculating per day cost of day and year notarizations */
	//costText2.setMaximumSize(new Dimension(50,45));
	//costText2.setMinimumSize(new Dimension(50,45));
	costPanel1.add(costText1);
	    /* Michael Patterson 10/08/2008 - Selecting to display in Units or $ */
		String sNotarizationPerYear;
		if(bUseUnits)
		{
			sNotarizationPerYear = "Per Year (in Units): ";
		}
		else
		{
			sNotarizationPerYear = "Per Year $: ";
		}
		//costPanel1.add(new JLabel("  Per Year: $"));
		costPanel1.add(new JLabel(sNotarizationPerYear));
	    /* MP - End changes for display in Units or $ */
	costPanel1.add(costText2);
	    /* Michael Patterson 10/08/2008 - Selecting to display in Units or $ */
		String sFAOnek;
		if(bUseUnits)
		{
			sFAOnek = "One Corruption (in Units):";
		}
		else
		{
			sFAOnek = "One Corruption $:";
		}
		//costPanel2.add(new JLabel("One Corruption: $"));
		costPanel2.add(new JLabel(sFAOnek));
	    /* MP - End changes for display in Units or $ */
		/* Michael Patterson 10/12/08 - Adding field for Forensic Analysis prediction */
		//costPanel2.add(new JTextField("2.25"));	
		costPanel2.add(singleFA);
		/* MP - End Forensic Analysis prediction field */
        costPanel2.add(Box.createRigidArea(new Dimension(20, 25)));
        CorruptionNumDropDown.setMaximumSize(new Dimension(50,25));
        CorruptionNumDropDown.setMinimumSize(new Dimension(50,25));
        /* Michael Patterson 10/12/08 - Adding ActionListener to field */
        CorruptionNumDropDown.addActionListener(aCalcFC);
        /* MP - End adding ActionListener */
        costPanel2.add(CorruptionNumDropDown);
	    /* Michael Patterson 10/08/2008 - Selecting to display in Units or $ */
		String sFAMultik;
		if(bUseUnits)
		{
			sFAMultik = "Corruptions (in Units):";
		}
		else
		{
			sFAMultik = "Corruptions $:";
		}
		//costPanel2.add(new JLabel("Corruptions: $"));
		costPanel2.add(new JLabel(sFAMultik));
	    /* MP - End changes for display in Units or $ */   
		/* Michael Patterson 10/12/08 - Adding field for Forensic Analysis prediction */		
        //costPanel2.add(new JTextField("200.34"));	
		costPanel2.add(multiFA);
    	/* MP - End Forensic Analysis prediction field */
        CostPanel.add(CostLabel1);
        CostPanel.add(Box.createRigidArea(new Dimension(5, 10)));
        CostPanel.add(CostLabel2);
        CostPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        CostPanel.add(costPanel1);
        CostPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        CostPanel.add(CostLabel3);
        CostPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        CostPanel.add(costPanel2);
        CostPanel.add(Box.createRigidArea(new Dimension(5, 0)));


        datePanel.setLayout(new BoxLayout(datePanel, BoxLayout.X_AXIS));
        JLabel dateLabel = new JLabel("Start On:");
        dateLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        datePanel.add(dateLabel);
        datePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        month.setMaximumSize(new Dimension(50,30));
        month.setMinimumSize(new Dimension(50,30));
        datePanel.add(month);
        JLabel dashLabel = new JLabel("/");
        dashLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        datePanel.add(dashLabel);
        datePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        day.setMaximumSize(new Dimension(50,35));
        day.setMinimumSize(new Dimension(50,35));
        datePanel.add(day);
        JLabel dashLabel2 = new JLabel("/");
        dashLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        datePanel.add(dashLabel2);
        datePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        year.setMaximumSize(new Dimension(65,30));
        year.setMinimumSize(new Dimension(65,30));
        datePanel.add(year);
        JLabel atLabel = new JLabel("   at   ");
        atLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        datePanel.add(atLabel);
        datePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        hour.setMaximumSize(new Dimension(70,30));
        hour.setMinimumSize(new Dimension(70,30));
        datePanel.add(hour);
        JLabel colonLabel = new JLabel(" : ");
        colonLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        datePanel.add(colonLabel);
        datePanel.add(Box.createRigidArea(new Dimension(5, 0)));
        minute.setMaximumSize(new Dimension(50,30));
        minute.setMinimumSize(new Dimension(50,30));
        datePanel.add(minute);
        datePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
        datePanel.add(Box.createVerticalGlue());

        savePanel.setLayout(new BoxLayout(savePanel, BoxLayout.Y_AXIS));
        saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        savePanel.add(saveButton);
        savePanel.add(Box.createRigidArea(new Dimension(200, 5)));

        problemsPanel.setLayout(new BoxLayout(problemsPanel, BoxLayout.Y_AXIS));
        JLabel activitiesLabel = new JLabel("Detected Tampering:");
        activitiesLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        problemsPanel.add(activitiesLabel);
        problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        scrollingActivitiesList = new JScrollPane(activitiesList);
        problemsPanel.add(scrollingActivitiesList);

        JLabel FACostSingleTamperLabel = new JLabel("Forensic Analysis cost to analyze this tampering is  $ ");
	FACostSingleTamperLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        FAcostPanel1.setLayout(new BoxLayout(FAcostPanel1, BoxLayout.X_AXIS));
        FAcostPanel1.add(FACostSingleTamperLabel);
        FAcostText1.setText("1.25");
        FAcostText1.setMaximumSize(new Dimension(50,30));
        FAcostText1.setMinimumSize(new Dimension(50,30));
        FAcostPanel1.add(FAcostText1);
	FAcostPanel1.setAlignmentX(Component.CENTER_ALIGNMENT);


	problemsPanel.add(FAcostPanel1);
        JLabel FACostSingleDBLabel = new JLabel("Forensic Analysis cost to analyze this database is  $ ");
	//FACostSingleDBLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        FAcostPanel2.setLayout(new BoxLayout(FAcostPanel2, BoxLayout.X_AXIS));
        FAcostPanel2.add(FACostSingleDBLabel);
        FAcostText2.setText("2.25");
        FAcostText2.setMaximumSize(new Dimension(50,30));
        FAcostText2.setMinimumSize(new Dimension(50,30));
        FAcostPanel2.add(FAcostText2);
	FAcostPanel2.setAlignmentX(Component.CENTER_ALIGNMENT);

	problemsPanel.add(FAcostPanel2);
        JLabel FACostAllDBLabel = new JLabel("Forensic Analysis cost to analyze all my databases is  $ ");
	//FACostAllDBLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        FAcostPanel3.setLayout(new BoxLayout(FAcostPanel3, BoxLayout.X_AXIS));
        FAcostPanel3.add(FACostAllDBLabel);
        FAcostText3.setText("3.25");
        FAcostText3.setMaximumSize(new Dimension(50,30));
        FAcostText3.setMinimumSize(new Dimension(50,30));
        FAcostPanel3.add(FAcostText3);
	FAcostPanel3.setAlignmentX(Component.CENTER_ALIGNMENT);

	problemsPanel.add(FAcostPanel3);
        problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        viewBtn.setAlignmentX(Component.CENTER_ALIGNMENT);
        problemsPanel.add(viewBtn);
        problemsPanel.setAlignmentX(Component.CENTER_ALIGNMENT);




        saveButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ae)
            {
                saveSettings();
            }
        });
        
        boxPanel.setLayout(new BoxLayout(boxPanel,BoxLayout.Y_AXIS));
        boxPanel.add(notPanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(validPanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(NotorizationIntervalPanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(NotorizationTimePanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(ValidationIntervalPanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(ValidationTimePanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        /* Michael Patterson 10/12/08 - Adding panel for Cost Per Unit */
        if(!bUseUnits)
        {
        	boxPanel.add(CostPerUnitPanel);
        	boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        }
        /* MP - End adding of cost per unit panel */
        separator1.setPreferredSize(new Dimension(600,0));
        boxPanel.add(separator1);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
	boxPanel.add(CostPanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        separator2.setPreferredSize(new Dimension(600,0));
        boxPanel.add(separator2);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(datePanel);
        boxPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        boxPanel.add(savePanel);


        pane.addTab("Settings", boxPanel);
        pane.addTab("Detected Tampering", problemsPanel);

        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setLayout(new BorderLayout());
        frame.add(topPanel, BorderLayout.NORTH);
        frame.add(pane, BorderLayout.CENTER);
        frame.setSize(600,550);
	frame.setLocation(screenWt/3,screenHt/3);
        frame.setVisible(true);

        //Make sure values are loaded/selected in the GUI
        update(null, null);
    }



    /**
     * Retrieves the list of all databases that the current DBA is
     * assigned to (from the model) and populates the database
     * drop down at the top of the application.
     */
    private void loadDatabases()
    {
        Iterator iter = model.getManagedDatabases().iterator();
        databaseDropDown.addItem("All Databases");
        while(iter.hasNext())
        {
            databaseDropDown.addItem(iter.next().toString());
        }
    }

    /*
     * Refreshes all the lists and drop down boxes in the application and
     * selects the current values.
     *
     * (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable arg0, Object arg1)
    {
        try
        {
            //Select the current database
            databaseDropDown.setSelectedItem(model.getSelectedDB());

            //Display the CSI
            csiLabel.setText("CSI: " + model.getCurrentCSI());

            //Get the config settings
            DatabaseConfig config = model.getCurrentConfig();
            //notzDaysDropDown.setSelectedItem(config.getNotzIntervalDays());
            notzDaysDropDown.setSelectedItem("0");
            notzHoursDropDown.setSelectedItem(config.getNotzIntervalHours());
            notzMinutesDropDown.setSelectedItem(config.getNotzIntervalMins());
            valSlider.setValue(config.getValidationFactor());
	    //valSlider.setValue(1);
            algorithmDropDown.setSelectedItem(config.getAlgorithm());
	    //VNDropDown.setSelectedItem("1");
            String dateTime = config.getStartDate();
            if(dateTime != null)
            {
                if(!dateTime.equals(""))
                {
                    //Year
                    year.setSelectedItem(Integer.valueOf(dateTime.substring(0, 4)));

                    //Month
                    if(dateTime.substring(5, 7).startsWith("0"))
                    {   //Single digit
                        month.setSelectedItem(Integer.valueOf(dateTime.substring(6, 7)));
                    }
                    else    //Double digit
                    {
                        month.setSelectedItem(Integer.valueOf(dateTime.substring(5, 7)));
                    }

                    //Day
                    if(dateTime.substring(8, 10).startsWith("0"))
                    {   //Single digit
                        day.setSelectedItem(Integer.valueOf(dateTime.substring(8, 10)));
                    }
                    else    //Double digit
                    {
                        day.setSelectedItem(Integer.valueOf(dateTime.substring(9, 10)));
                    }

                    //Hour
                    if(dateTime.substring(11, 13).startsWith("0"))
                    {   //Single digit
                        hour.setSelectedItem(Integer.valueOf(dateTime.substring(11, 13)));
                    }
                    else    //Double digit
                    {
                        hour.setSelectedItem(Integer.valueOf(dateTime.substring(12, 13)));
                    }

                    //Minute
                    if(dateTime.substring(14, 16).startsWith("0"))
                    {   //Single digit
                        minute.setSelectedItem(Integer.valueOf(dateTime.substring(14, 16)));
                    }
                    else    //Double digit
                    {
                        minute.setSelectedItem(Integer.valueOf(dateTime.substring(15, 16)));
                    }
                }
            }

            //Get the tamper detections
            activitiesListModel.clear();
            Iterator iter = model.getFailures().iterator();
            while(iter.hasNext())
            {
                activitiesListModel.addElement(iter.next());
            }
            scrollingActivitiesList.validate();

        }
        catch(Exception e)
        {}
    }


    /**
     * Saves the configuration settings for the currently selected database.
     */
    @SuppressWarnings("deprecation")
    private void saveSettings()
    {
        if(databaseDropDown.getSelectedItem() != null)
        {
            //Create a new database configuration
            DatabaseConfig config = new DatabaseConfig(
                    algorithmDropDown.getSelectedItem().toString(),
                    Integer.parseInt(notzDaysDropDown.getSelectedItem().toString()),
                    Integer.parseInt(notzHoursDropDown.getSelectedItem().toString()),
                    Integer.parseInt(notzMinutesDropDown.getSelectedItem().toString()),
                    1);
                    //valSlider.getValue()); 
 
/*

            //Create the ScheduledDate

            String strMonth =  month.getSelectedItem().toString();
            if(strMonth.equals("Jan"))
            {
                strMonth = "01";
            }
            if(strMonth.equals("Feb"))
            {
                strMonth = "02";
            }
            if(strMonth.equals("Jan"))
            {
                strMonth = "01";
            }
            if(strMonth.equals("Mar"))
            {
                strMonth = "03";
            }
            if(strMonth.equals("Apr"))
            {
                strMonth = "04";
            }
            if(strMonth.equals("May"))
            {
                strMonth = "05";
            }
            if(strMonth.equals("Jun"))
            {
                strMonth = "06";
            }
            if(strMonth.equals("Jul"))
            {
                strMonth = "07";
            }
            if(strMonth.equals("Aug"))
            {
                strMonth = "08";
            }
            if(strMonth.equals("Sep"))
            {
                strMonth = "09";
            }
            if(strMonth.equals("Oct"))
            {
                strMonth = "10";
            }
            if(strMonth.equals("Nov"))
            {
                strMonth = "11";
            }
            if(strMonth.equals("Dec"))
            {
                strMonth = "12";
            }

            String strDay =  day.getSelectedItem().toString();
            if(strDay.length() == 1)
            {
                strDay = "0" + strDay;
            }


            String strHour =  hour.getSelectedItem().toString();
            if(strHour.length() == 3)
            {
                strHour = "0" + strHour.substring(0, 1);
            }
            if(strHour.length() == 4)
            {
                strHour = strHour.substring(0, 2);
            }

            String strMin = minute.getSelectedItem().toString();

	    ///* Don't need that. It's now "03". 
            if(strMin.length() == 1)
            {
                strMin = "0" + strMin;
            }
	    //*/



            String strMonth =  month.getSelectedItem().toString();
            if(strMonth.length() == 1)
            {
                strMonth = "0" + strMonth;
            }
            String strDay =  day.getSelectedItem().toString();
            if(strDay.length() == 1)
            {
                strDay = "0" + strDay;
            }
            String strHour =  hour.getSelectedItem().toString();
            if(strHour.length() == 1)
            {
                strHour = "0" + strHour;
            }
            String strMin = minute.getSelectedItem().toString();
            if(strMin.length() == 1)
            {
                strMin = "0" + strMin;
            }


            config.setStartDate(year.getSelectedItem().toString() + "-" +
                    strMonth + "-" + strDay + " " +
                    strHour + ":" + strMin + ":00");
            //Tell the model to save the settings
            model.saveDBConfig(config);

        }
        else
        {
            JOptionPane.showMessageDialog(null, "Settings cannot be saved because there is not a database selected.", "No databases", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    /* Michael Patterson 10/12/08 - This function calculates the Forensic Cost based on the values in the
     * Settings tab.
     */
    private void calculateFC()
    {
        ForensicCost fc = new ForensicCost();
        DecimalFormat dcForm = new DecimalFormat();
        Double costPerDay = 0.0;
        Double costPerYear = 0.0;
        Double dSingleFA = 0.0;
        Double dMultiFA = 0.0;
        
        // Determine length of decimal based on whether displaying in Units or $
        if(bUseUnits)
        {
        	// Set the decimal length
	        dcForm.setMaximumFractionDigits(0);
	        dcForm.setMinimumFractionDigits(0);
        }
        else
        {
        	// Set the decimal length
	        dcForm.setMaximumFractionDigits(2);
	        dcForm.setMinimumFractionDigits(2);
        }

		// Get the values of the fields in terms of Units
    	// R_s value
        Double notzResolution = (Double.parseDouble(notzDaysDropDown.getSelectedItem().toString()) * 1440.0) + 
		 				 		(Double.parseDouble(notzHoursDropDown.getSelectedItem().toString()) * 24.0) +
		 				 		 Double.parseDouble(notzMinutesDropDown.getSelectedItem().toString());
        // I_N value
        Double notzInterval = (Double.parseDouble(notDaysTextField.getText()) * 1440.0) + 
					   		  (Double.parseDouble(notHoursTextField.getText()) * 24.0) +
					   		   Double.parseDouble(notMinsTextField.getText());
        // I_V value
        Double valInterval = (Double.parseDouble(notDaysTextField1.getText()) * 1440.0) + 
		 			  		 (Double.parseDouble(notHoursTextField1.getText()) * 24.0) +
		 			  		  Double.parseDouble(notMinsTextField1.getText());
        // N value
        Integer numNotzResUnits = Integer.parseInt(NDropDown.getSelectedItem().toString());
        // V value
        Integer numValNotzUnits = Integer.parseInt(VNDropDown.getSelectedItem().toString());
        // k value
        Integer kval = Integer.parseInt(CorruptionNumDropDown.getSelectedItem().toString());
        // Cost per unit value
        Double costPerUnit = Double.parseDouble(costPerUnitField.getText());

        //Determine the Forensic Cost Algorithm to use
        if(algorithmDropDown.getSelectedItem().toString().compareTo("Monochromatic") == 0)
        {
        	costPerDay = fc.NP_mono(notzResolution, numValNotzUnits);
        	costPerYear = fc.NP_mono(notzResolution, numValNotzUnits) * 365;
    		// Monochromatic value for Forensic Analysis Prediction
        	dSingleFA = fc.FA_mono(notzResolution);
        	if(kval == 0)
        	{
        		dMultiFA = 0.0;
        	}
        	else
        	{
        		dMultiFA = fc.FA_mono(notzResolution);
        	}
        }
        else if(algorithmDropDown.getSelectedItem().toString().compareTo("RGBY") == 0)
        {
        	costPerDay = fc.NP_RGBY(notzResolution);
        	costPerYear = fc.NP_RGBY(notzResolution) * 365;
        	// RGBY value for Forensic Analysis Prediction
        	dSingleFA = fc.FA_RGBY(notzResolution);
        	if(kval == 0)
        	{
        		dMultiFA = 0.0;
        	}
        	else
        	{
        		dMultiFA = fc.FA_RGBY(notzResolution);
        	}
        }
        else if(algorithmDropDown.getSelectedItem().toString().compareTo("Tiled Bitmap") == 0)
        {
        	costPerDay = fc.NP_tiled(notzResolution, numNotzResUnits, numValNotzUnits);
        	costPerYear = fc.NP_tiled(notzResolution, numNotzResUnits, numValNotzUnits) * 365;
        	// Tiled Bitmap value for Forensic Analysis Prediction
        	dSingleFA = fc.FA_tiled(notzResolution, numNotzResUnits, 1);
        	if(kval == 0)
        	{
        		dMultiFA = 0.0;
        	}
        	else
        	{
        		dMultiFA = fc.FA_tiled(notzResolution, numNotzResUnits, kval);
        	}
        }
        else if(algorithmDropDown.getSelectedItem().toString().compareTo("A3D") == 0)
        {
        	costPerDay = fc.NP_a3D(notzResolution, numNotzResUnits);
        	costPerYear = fc.NP_a3D(notzResolution, numNotzResUnits) * 365;
        	// A3D value for Forensic Analysis Prediction
        	dSingleFA = fc.FA_a3D(notzResolution, 1);
        	if(kval == 0)
        	{
        		dMultiFA = 0.0;
        	}
        	else
        	{
        		dMultiFA = fc.FA_a3D(notzResolution, kval);
        	}
        }
        
        // Set the cost values to be displayed
        if(bUseUnits)
        {
	        costText1.setText(dcForm.format(costPerDay));
	        costText2.setText(dcForm.format(costPerYear));
	        singleFA.setText(dcForm.format(dSingleFA));
	        multiFA.setText(dcForm.format(dMultiFA));
        }
        else
        {
	        costText1.setText(dcForm.format(costPerDay * costPerUnit));
	        costText2.setText(dcForm.format(costPerYear * costPerUnit));
	        singleFA.setText(dcForm.format(dSingleFA * costPerUnit));
	        multiFA.setText(dcForm.format(dMultiFA * costPerUnit));
        }
        
    }
    /* MP - End calculateFC for calculating Forensic Cost based on the field values on the Settings tab. */
}


