package gui;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.*;

/**
 * Displays the login screen for the Database Monitor Application.
 * If the employee can be verified in the Security Master Database
 *  as a valid employee the CSI application is launched.
 * 
 * @author Natasha Gaitonde - gaitonde@email.arizona.edu
 *
 */
@SuppressWarnings("serial") 
class CSILoginDialog extends JDialog implements ActionListener
{
    private JTextField email;
    private JPasswordField password;
    Toolkit kit = Toolkit.getDefaultToolkit();
	Dimension screenSize = kit.getScreenSize();
	int screenHt = screenSize.height;
	int screenWt = screenSize.width;

    /**
     * Sets up the GUI components for the dialog box
     */
    public CSILoginDialog()
    {
        JLabel emailLabel = new JLabel("Email:", JLabel.TRAILING);
        email = new JTextField(50);

        JLabel passwordLabel = new JLabel("Password:", JLabel.TRAILING);
        password = new JPasswordField(50);

        JButton loginButton = new JButton("Login");
        loginButton.addActionListener(this);

        JButton exitButton = new JButton("Exit");
        exitButton.addActionListener(this);

        setTitle("Login to CSI Application");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        //Create and populate panel
        JPanel topPanel = new JPanel(new SpringLayout());
        topPanel.add(emailLabel);
        topPanel.add(email);
        topPanel.add(passwordLabel);
        topPanel.add(password);
        topPanel.add(exitButton);
        topPanel.add(loginButton);

        //Lay out the panel.
        SpringUtilities.makeCompactGrid(topPanel,
                3, 2, //rows, cols
                6, 6, //initX, initY
                6, 6);//xPad, yPad

        //Add the panel to this JDialog
        setContentPane(topPanel);
        setSize(screenWt/4,screenHt/4);
		setLocation(screenWt/4,screenHt/3);
        pack();
    }

    /** 
     * Responds to the Login and Exit buttons.
     * On "Exit" it simply exits the program.
     * On "Login" then the employee is checked in the database and is valid,
     * the CSI application is then launched. Otherwise an appropriate error
     * message is displayed to the user.
     */
    public void actionPerformed(ActionEvent ae)
    {
        if(ae.getActionCommand().equals("Login"))
        {
            //Check the database to see if this login is valid
            Connection conn = Connector.getConnection();
            String query = "select Name from Employee where Email = '" + email.getText() + "' and Password='" + String.valueOf(password.getPassword()) + "' and isCSI='Y';";
            if(conn != null)
            {
                try
                {
                    Statement stmt = conn.createStatement();
                    ResultSet rset = stmt.executeQuery(query);

                    if(rset.next()) //This login is valid
                    {
/* Modified by Huilong Huang on 5/11/2006 */
                        String name=rset.getString("Name");
                        new CSIView(name, email.getText());
//                        new CSIView(email.getText());
/* End of modification */
                        this.setVisible(false);
                        this.dispose();
                    }
                    else //Invalid login
                    {
                        JOptionPane.showMessageDialog(null, "The email and password did not match or the user is not designated as a CSI.", "Invalid Login", JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                }
                catch (SQLException se)
                {
                    System.err.println("SQLException while attempting to login.");
                    se.printStackTrace();
                }
                catch(Exception e)
                {
                    System.err.println("Exception while attempting to login.");
                    e.printStackTrace();
                }
            }
        }
        else if(ae.getActionCommand().equals("Exit"))
        {
            System.exit(0);
        }
    }

    /**
     * Invokes the CSI Application with a Login screen
     */
    public static void main(String[] args)
    {
        CSILoginDialog dialog = new CSILoginDialog();
        dialog.setVisible(true);
    }
}
