package gui;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Timestamp;
import java.util.*;


import javax.swing.*;



/**
 * Main screen for the CSI. It is after the Login Screen.
 * At the top we have a drop down for the Databases managed by the CSI. Alongside we have a field the the DBA Name.
 * Below that the configuration for the selected Database is show. If the Database has no individual configuration, 
 * then the overall configuration is displayed. The CSI can make changes to the Configuration and save them by
 * clicking the Change Configuration button.
 * At the bottom is the button to view the Events on a GNUPlot. This plot shows the Notarization and Validation
 * Events. If any corruption regions are present, then they are depicted as rectangles.
 * 
 * @author Natasha Gaitonde - gaitonde@email.arizona.edu
 *
 */
public class CSIView implements Observer
{
	private CSIModel model = new CSIModel();
	private CSIPlotMain plot = null;
	DatabaseConfig confignew, config;

	//Main frame
	private JFrame frame = new JFrame("CSI Application");
	private JPanel topPanel = new JPanel();
	private JPanel viewPanel = new JPanel();
    //for configuration
	private JPanel dbPanel3 = new JPanel();
	private JPanel settingsPanel1 = new JPanel();
	private JPanel settingsPanel2 = new JPanel();
	private JPanel settingsPanel3 = new JPanel();
    //Tamper Detection Components (problemsPanel components)
        private JPanel problemsPanel = new JPanel();
        private DefaultListModel activitiesListModel = new DefaultListModel();
        private JList activitiesList = new JList(activitiesListModel);
        private JScrollPane scrollingActivitiesList;

	
	private JComboBox databaseDropDown = new JComboBox();
	private JLabel dbaLabel = new JLabel("DBA:  Susan Miller(sjmiller@email.arizona.edu)");
	private JButton viewBtn = new JButton("View Database Corruption Graphically");
	private JMenuBar menuBar = new JMenuBar();
	private JMenu menu = new JMenu("File");
	private JMenuItem exitItem = new JMenuItem("Exit");
	Toolkit kit = Toolkit.getDefaultToolkit();
	Dimension screenSize = kit.getScreenSize();
	int screenHt = screenSize.height;
	int screenWt = screenSize.width;

	String[] algs = {"Monochromatic", "RGBY", "Tiled Bitmap", "A3D"};
	private JComboBox algorithmDropDown = new JComboBox(algs);
	private JTextField notzDays = new JTextField();
	Integer[] hours = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};
	private JComboBox notzHoursDropDown = new JComboBox(hours);
	Integer[] mins = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59};
	private JComboBox notzMinutesDropDown = new JComboBox(mins);
	Integer[] validInt = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
	private JComboBox valIntervalDropDown = new JComboBox(validInt);
	private JTextField level = new JTextField();
	private JButton saveButton = new JButton("Change Configuration");
        private JTextField FAcostText1 = new JTextField();
        private JTextField FAcostText2 = new JTextField();    
        private JTextField FAcostText3 = new JTextField();
        private JPanel FAcostPanel1 = new JPanel();
        private JPanel FAcostPanel2 = new JPanel();
        private JPanel FAcostPanel3 = new JPanel();

/* Modified by Huilong Huang on 5/11/2007 */
    public CSIView(String name, String csiEmail)
//	public CSIView(String csiEmail)
    {
        frame.setTitle("CSI: "+name+" Email:"+csiEmail);
/* End Huilong */
		if(csiEmail != null)
		{
			model.setCurrentCSI(csiEmail);
		}
		model.addObserver(this);
		loadDatabases();//get managed databases for the CSI
		menuBar.add(menu);
		exitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0)
			{
				System.exit(0);
			}
		});
		menu.add(exitItem);
		frame.setJMenuBar(menuBar);

		//Get configuation data for the selected database in the dropdown
		config = model.getCurrentConfig();
		if (config==null)
		{
			System.out.println("Error: No Configuration for Database " + model.getCurrentDBName() + " or Overall");
            JOptionPane.showMessageDialog(null, "The email and password did not match or the user is not designated as a CSI.", "Invalid Login", JOptionPane.ERROR_MESSAGE);
            System.exit(1);
		}
		//populate the data in the elements
		notzDays.setText(Integer.toString(config.getNotzIntervalDays()));
		notzHoursDropDown.setSelectedItem(config.getNotzIntervalHours());
		notzMinutesDropDown.setSelectedItem(config.getNotzIntervalMins());
		valIntervalDropDown.setSelectedItem(config.getValidationFactor());
		algorithmDropDown.setSelectedItem(config.getAlgorithm());
		level.setText(Integer.toString(config.getLevel()));

		databaseDropDown.setAlignmentX(Component.CENTER_ALIGNMENT);
		databaseDropDown.setMaximumSize(new Dimension(200,30));
		databaseDropDown.setMinimumSize(new Dimension(200,30));
                topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
		topPanel.add(databaseDropDown);
		topPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		dbaLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		topPanel.add(dbaLabel);
		viewPanel.setLayout(new FlowLayout());
		viewPanel.add(viewBtn);
		databaseDropDown.addActionListener(new ActionListener() 
		{
			public void actionPerformed(ActionEvent ae)
			{
				if(databaseDropDown.getSelectedItem() != null)
				{
					try
					{
						String selectedText = databaseDropDown.getSelectedItem().toString();
						StringTokenizer tokenizer = new StringTokenizer(selectedText, " ()");
						model.setSelectedDB(tokenizer.nextToken(), tokenizer.nextToken());
						update(null, null);
						//Update configuration for the selected database
						config = model.getCurrentConfig();
						if (config==null)
						{
							System.out.println("Error: Database " + model.getCurrentDBName() + " is not audited");
							System.exit(1);
						}
						notzDays.setText(Integer.toString(config.getNotzIntervalDays()));
						notzHoursDropDown.setSelectedItem(config.getNotzIntervalHours());
						notzMinutesDropDown.setSelectedItem(config.getNotzIntervalMins());
						valIntervalDropDown.setSelectedItem(config.getValidationFactor());
						algorithmDropDown.setSelectedItem(config.getAlgorithm());
						level.setText(Integer.toString(config.getLevel()));
					}
					catch(Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		});

		//Layout of all display panels and adding individual elements
		settingsPanel1.setLayout(new BoxLayout(settingsPanel1, BoxLayout.X_AXIS));
		JLabel algorithmLabel = new JLabel("Algorithm:");
		algorithmLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel1.add(algorithmLabel);
		settingsPanel1.add(Box.createRigidArea(new Dimension(5, 0)));
		algorithmDropDown.setMaximumSize(new Dimension(150,30));
		algorithmDropDown.setMinimumSize(new Dimension(150,30));
		settingsPanel1.add(algorithmDropDown);
		settingsPanel1.setAlignmentX(Component.CENTER_ALIGNMENT);

		settingsPanel2.setLayout(new BoxLayout(settingsPanel2, BoxLayout.X_AXIS));
		JLabel notarizationLabel = new JLabel("Max Notarization Interval:");
		notarizationLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel2.add(notarizationLabel);
		settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
		notzDays.setMaximumSize(new Dimension(50,30));
		notzDays.setMinimumSize(new Dimension(50,30));
		settingsPanel2.add(notzDays);
		settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
		JLabel dayLabel = new JLabel("days");
		dayLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel2.add(dayLabel);
		settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
		notzHoursDropDown.setMaximumSize(new Dimension(50,30));
		notzHoursDropDown.setMinimumSize(new Dimension(50,30));
		settingsPanel2.add(notzHoursDropDown);
		settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
		JLabel hourLabel = new JLabel("hrs");
		hourLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel2.add(hourLabel);
		settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
		notzMinutesDropDown.setMaximumSize(new Dimension(50,30));
		notzMinutesDropDown.setMinimumSize(new Dimension(50,30));
		settingsPanel2.add(notzMinutesDropDown);
		settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
		JLabel minLabel = new JLabel("mins.");
		minLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel2.add(minLabel);
		settingsPanel2.setAlignmentX(Component.CENTER_ALIGNMENT);

		settingsPanel3.setLayout(new BoxLayout(settingsPanel3, BoxLayout.X_AXIS));
		JLabel validationLabel = new JLabel("Validation Factor:");
		validationLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel3.add(validationLabel);
		settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
		valIntervalDropDown.setMaximumSize(new Dimension(50,30));
		valIntervalDropDown.setMinimumSize(new Dimension(50,30));
		settingsPanel3.add(valIntervalDropDown);
		JLabel levelLabel = new JLabel("Level:");
		levelLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		settingsPanel3.add(levelLabel);
		settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
		level.setMaximumSize(new Dimension(50,30));
		level.setMinimumSize(new Dimension(50,30));
		settingsPanel3.add(level);        
		settingsPanel3.setAlignmentX(Component.CENTER_ALIGNMENT);


		problemsPanel.setLayout(new BoxLayout(problemsPanel, BoxLayout.Y_AXIS));
		JLabel activitiesLabel = new JLabel("Detected Tampering:");
		activitiesLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		problemsPanel.add(activitiesLabel);
		problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
		scrollingActivitiesList = new JScrollPane(activitiesList);
		problemsPanel.add(scrollingActivitiesList);
 
		JLabel FACostSingleTamperLabel = new JLabel("Forensic Analysis cost to analyze this tampering is  $ ");
		FACostSingleTamperLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		FAcostPanel1.setLayout(new BoxLayout(FAcostPanel1, BoxLayout.X_AXIS));
		FAcostPanel1.add(FACostSingleTamperLabel);
		FAcostText1.setText("1.25");
		FAcostText1.setMaximumSize(new Dimension(50,30));
		FAcostText1.setMinimumSize(new Dimension(50,30));
		FAcostPanel1.add(FAcostText1);


		problemsPanel.add(FAcostPanel1);
		JLabel FACostSingleDBLabel = new JLabel("Forensic Analysis cost to analyze this database is  $ ");
		FACostSingleDBLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		FAcostPanel2.setLayout(new BoxLayout(FAcostPanel2, BoxLayout.X_AXIS));
		FAcostPanel2.add(FACostSingleDBLabel);
		FAcostText2.setText("2.25");
		FAcostText2.setMaximumSize(new Dimension(50,30));
		FAcostText2.setMinimumSize(new Dimension(50,30));
		FAcostPanel2.add(FAcostText2);


		problemsPanel.add(FAcostPanel2);
		JLabel FACostAllDBLabel = new JLabel("Forensic Analysis cost to analyze all my databases is  $ ");
		FACostAllDBLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		FAcostPanel3.setLayout(new BoxLayout(FAcostPanel3, BoxLayout.X_AXIS));
		FAcostPanel3.add(FACostAllDBLabel);
		FAcostText3.setText("3.25");
		FAcostText3.setMaximumSize(new Dimension(50,30));
		FAcostText3.setMinimumSize(new Dimension(50,30));
		FAcostPanel3.add(FAcostText3);

		problemsPanel.add(FAcostPanel3);
		problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));


		dbPanel3.setLayout(new BoxLayout(dbPanel3, BoxLayout.Y_AXIS));
		dbPanel3.add(settingsPanel1);
		dbPanel3.add(settingsPanel2);
		dbPanel3.add(settingsPanel3);
		saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		dbPanel3.add(saveButton);

		saveButton.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent ae)
			{
				//Save the configuration
				int refresh=0;
				try
				{
					Calendar cal=Calendar.getInstance();
					confignew = new DatabaseConfig(
							model.getCurrentDBName(),
							algorithmDropDown.getSelectedItem().toString(),
							Integer.parseInt(notzDays.getText()),
							Integer.parseInt(notzHoursDropDown.getSelectedItem().toString()),
							Integer.parseInt(notzMinutesDropDown.getSelectedItem().toString()),
							Integer.parseInt(valIntervalDropDown.getSelectedItem().toString()),
							Integer.parseInt(level.getText()),
							new Timestamp(cal.getTime().getTime()),
							"0000-00-00 00:00:00");
					refresh=model.setConfig(config,confignew);//check if config was updated
					if(refresh==1)
						config=confignew;
				}
				catch(NumberFormatException ne)
				{
					JOptionPane.showMessageDialog(null, "Please enter a numeric value for the notarization interval in days.", "Non-numeric value", JOptionPane.ERROR_MESSAGE);
				}
				catch(Exception e)
				{}

			}
		});

		frame.setLayout(new BorderLayout());
		frame.add(topPanel,BorderLayout.NORTH);
	        frame.add(problemsPanel,BorderLayout.CENTER);
 //		frame.add(dbPanel3,BorderLayout.CENTER);
		frame.add(viewPanel,BorderLayout.SOUTH);
		frame.setVisible(true);
		frame.setSize(screenWt/3,screenHt/3);
		frame.setLocation(screenWt/3,screenHt/3);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		update(null, null);

		viewBtn.addActionListener(new ActionListener() 
		{
			public void actionPerformed(ActionEvent ae)
			{
				if(databaseDropDown.getSelectedItem() != null)
				{
					try
					{
						String selectedText = databaseDropDown.getSelectedItem().toString();
						StringTokenizer tokenizer = new StringTokenizer(selectedText, " ()");
						model.setSelectedDB(tokenizer.nextToken(), tokenizer.nextToken());
						update(null, null);
						plot = new CSIPlotMain(model.getCurrentDBName(),model.getCurrentDBPath());
                        plot.drawEvents();
					}
					catch(Exception e)
					{
						e.printStackTrace();
					}
				}
			}
		});        
	}
	private void loadDatabases()
	{
		Iterator iter = model.getManagedDatabases().iterator();
                databaseDropDown.addItem("All Databases");
		while(iter.hasNext())
		{
			databaseDropDown.addItem(iter.next().toString());
		}
	}
	public void update(Observable arg0, Object arg1)
	{
		try
		{

                        //dbaLabel.setText("DBA: " + model.getCurrentDBA());

			databaseDropDown.setSelectedItem(model.getSelectedDB());
			//dbaLabel.setText("DBA: " + model.getCurrentDBA());

       			//Tamper detections
        		activitiesListModel.clear();
        		Iterator iter = model.getAllFailures().iterator();
        		while(iter.hasNext())
        		{
       		     		activitiesListModel.addElement(iter.next());
        		}
        		scrollingActivitiesList.validate();


		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
