package gui;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.*;

/**
 * Displays the login screen for the Security Master Application.
 * If the employee can be verified in the Security Master Database
 *  as a valid employee the application is then launched.
 * 
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
class CSOLoginDialog extends JDialog implements ActionListener
{
    private boolean debug = false;
    private JTextField email;
    private JPasswordField password;
    Toolkit kit = Toolkit.getDefaultToolkit();
    Dimension screenSize = kit.getScreenSize();
    int screenHt = screenSize.height;
    int screenWt = screenSize.width;
    
    
    private String notarizer_server_;
    private String dbcreate_time_getter_;

    /**
     * Sets up the GUI components for the dialog box
     */
    public CSOLoginDialog()
    {

        JLabel emailLabel = new JLabel("Email:", JLabel.TRAILING);
        email = new JTextField(50);
        JLabel passwordLabel = new JLabel("Password:", JLabel.TRAILING);
        password = new JPasswordField(50);
        JButton loginButton = new JButton("Login");
        loginButton.addActionListener(this);
        JButton exitButton = new JButton("Exit");
        exitButton.addActionListener(this);

        setTitle("Login");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);


        //Create and populate panel
        JPanel topPanel = new JPanel(new SpringLayout());
        topPanel.add(emailLabel);
        topPanel.add(email);
        topPanel.add(passwordLabel);
        topPanel.add(password);
        topPanel.add(exitButton);
        topPanel.add(loginButton);

//      Lay out the panel.
        SpringUtilities.makeCompactGrid(topPanel,
                3, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad


        //Add the panel to this JDialog
        setContentPane(topPanel);
        setSize(screenWt/4,screenHt/4);
	setLocation(screenWt/4,screenHt/3);
        pack();
    }


    /* 
     * Responds to the Login and Exit buttons.
     * On "Exit" it simply exits the program.
     * On "Login" then the employee is checked in the database and is valid,
     * the application is then launched. Otherwise an appropriate error
     * message is displayed to the user.
     * 
     * (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent ae)
    {
        if(ae.getActionCommand().equals("Login"))
        {
            //Check the database to see if this login is valid
            Connection conn = Connector.getConnection();
            String query = "select Name from Employee where Email = '" + email.getText() + "' and Password='" + String.valueOf(password.getPassword()) + "' and isCSO='Y';";
            if(debug) { System.out.println(query); };
            if(conn != null)
            {
                try
                {
                    Statement stmt = conn.createStatement();
                    ResultSet rset = stmt.executeQuery(query);

/* Modified by Huilong Huang on 5/11/2006 */         
                    if(rset.next()) //This login is valid
                    {
                        String name=rset.getString("Name");
                        new CSOView(name, email.getText(), notarizer_server_, dbcreate_time_getter_);
/* End of modification */
                        //Close the login dialog
                        this.setVisible(false);
                        this.dispose();
                    }
                    else    //Invalid login
                    {
                        JOptionPane.showMessageDialog(null, "The email and password did not match or the user is not designated as the CSO.", "Invalid Login", JOptionPane.ERROR_MESSAGE);
                        return;
                    }

                }
                catch (SQLException se)
                {
                    System.err.println("SQLException while attempting to login.");
                    System.err.println(se.toString());
                }
                catch(Exception e)
                {
                    System.err.println("Exception while attempting to login.");
                }
            }

        }
        else if(ae.getActionCommand().equals("Exit"))
        {
            System.exit(0);
        }

    }
    
    public void setNotarizerServer(String notarizer_server) {
      notarizer_server_ = notarizer_server;
    }
      
    public void setDBCreatTimeGetter(String dbcreate_time_getter) {
      dbcreate_time_getter_ = dbcreate_time_getter;    	
    }
    


    /**
     * Invokes the Security Master Application
     */
    public static void main(String[] args)
    {
    	if (args.length != 2) {
    		Usage();
    		return;
    	}
        CSOLoginDialog dialog = new CSOLoginDialog();
        dialog.setNotarizerServer(args[0]);
        dialog.setDBCreatTimeGetter(args[1]);
        dialog.setVisible(true);
    }
    
    public static void Usage() {
      System.out.println(
          "Usage: java -jar CSO.jar notarizer_server dbcreate_time_getter");
    }

}
