package gui;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


/**
 * This is the Security Master Application (CSO GUI). It contains all the components
 * needed for the graphical user interface and interacts wth CSOModel.java to implement
 * all the features. 
 * This class should be invoked by the CSOLoginDialog.java class
 * 
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
public class CSOView implements Observer
{


    Toolkit kit = Toolkit.getDefaultToolkit();
    Dimension screenSize = kit.getScreenSize();
    int screenHt = screenSize.height;
    int screenWt = screenSize.width;
    //Turn on for debugging
    private boolean debug = false;

    private CSOModel cso_model_ = null;
    private String selectedDBName;
    private String selectedDBPath;

    private JFrame frame = new JFrame("Security Master Application (CSO View)");
    private JTabbedPane pane = new JTabbedPane();
    private JPanel databasePanel = new JPanel();
    private JPanel dbPanel1 = new JPanel();
    private JPanel dbPanel2 = new JPanel();
    private JPanel dbPanel3 = new JPanel();
    private JPanel dbaPanel = new JPanel();
    private JPanel dbaPanelLeft = new JPanel();
    private JPanel dbaPanelRight = new JPanel();
    private JPanel settingsPanel1 = new JPanel();
    private JPanel settingsPanel2 = new JPanel();
    private JPanel settingsPanel3 = new JPanel();
    private JPanel settingsPanel4 = new JPanel();
    private JPanel settingsPanel5 = new JPanel();
    private JPanel problemsPanel = new JPanel();
    private JPanel GlobalPanel = new JPanel();
    private JPanel EmployeePanel = new JPanel();
   
   
    //Databases tab variables
    private DefaultListModel listModel = new DefaultListModel();
    private JList dbList;
    private JScrollPane scrollingDBList;
    private JComboBox dbaDropDown;
    private JComboBox csiDropDown;
    private JButton assignButtonCSI = new JButton("Assign CSI");
    private JButton assignButtonDBA = new JButton("Assign DBA");
    String[] algs = {"Monochromatic", "RGBY", "Tiled Bitmap", "A3D"};
    private JComboBox algorithmDropDown = new JComboBox(algs);
    private JTextField notzDays = new JTextField();
    private JTextField notzDays1 = new JTextField("4");
    private JTextField CostSingleNorText = new JTextField();    
    private JTextField CostSingleNorDocText = new JTextField();
    Integer[] hours = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};
    private JComboBox notzHoursDropDown = new JComboBox(hours);
    private JComboBox notzHoursDropDown1 = new JComboBox(hours);
    Integer[] mins = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59};
    private JComboBox notzMinutesDropDown = new JComboBox(mins);
    private JComboBox notzMinutesDropDown1 = new JComboBox(mins);
    Integer[] validInt = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
    private JComboBox valIntervalDropDown = new JComboBox(validInt);
    private JButton saveButton = new JButton("Save");
    private JButton AddDatabaseBtn = new JButton("Add Database");
 


    //Employees tab variables
    private DefaultListModel dbaListModel = new DefaultListModel();
    private JList dbaList;
    private JScrollPane scrollingDBAList;
    private DefaultListModel managedListModel = new DefaultListModel();
    private DefaultListModel managedListModelforCSI = new DefaultListModel();
    private JList DBAformanagedList = new JList(managedListModel);
    private JList CSIformanagedList = new JList(managedListModelforCSI);
    private JScrollPane DBAforList;
    private JScrollPane CSIforList;
    private JButton AddEmployeeBtn = new JButton("Add Employee");


    //Problems tab variables
    private JComboBox databaseDropDown = new JComboBox();
    private DefaultListModel problemsListModel = new DefaultListModel();
    private JList problemsList = new JList(problemsListModel);
    private JScrollPane scrollingProblemsList;
    private JPanel PtopPanel = new JPanel();
    private JPanel PtopPanel1 = new JPanel();
    private JPanel PtopPanel2 = new JPanel();
    private JButton viewBtn = new JButton("View Database Corruption Graphically");
    private JTextField FAcostText1 = new JTextField();
    private JTextField FAcostText2 = new JTextField();    
    private JTextField FAcostText3 = new JTextField();
    private JPanel FAcostPanel1 = new JPanel();
    private JPanel FAcostPanel2 = new JPanel();
    private JPanel FAcostPanel3 = new JPanel();



    //Menu items
    private JMenuBar menuBar = new JMenuBar();
    private JMenu menu = new JMenu("File");
    private JMenuItem databaseItem = new JMenuItem("Add Database");
    private JMenuItem employeeItem = new JMenuItem("Add Employee");
    private JMenuItem exitItem = new JMenuItem("Exit");


    /**
     * Sets up and displays the application
     * 
     * @param dbaEmail the email of the currently logged in DBA
     */
/* Modified by Huilong Huang on 5/11/2007 */
    public CSOView(String name, String email, String notarize_server,
                   String dbcreate_time_getter) {
        frame.setTitle("CSO: "+name+" Email:"+email);
/* End Huilong */
        cso_model_ = new CSOModel();
        cso_model_.setNotarizeServer(notarize_server);
        cso_model_.setDBCreateTimeGetter(dbcreate_time_getter);
        //Add this view as an observer
        cso_model_.addObserver(this);
	    loadDatabases();//get managed databases for the CSI
        //-----------Setup menu-------------
        menuBar.add(menu);

        databaseItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                DatabaseDialog dbDialog = new DatabaseDialog(cso_model_);
                dbDialog.setVisible(true);
            }
        });
        menu.add(databaseItem);


        employeeItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                EmployeeDialog empDialog = new EmployeeDialog(cso_model_);
                empDialog.setVisible(true);
            }
        });
        menu.add(employeeItem);


        exitItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                System.exit(0);
            }
        });
        menu.add(exitItem);
        frame.setJMenuBar(menuBar);


        //-----------Populate fields-------------
        //Get the current settings from the model and populate the form correctly
        Iterator iter = cso_model_.getDatabases().iterator();
        while(iter.hasNext())
        {
            listModel.addElement(iter.next());
        }

        dbList = new JList(listModel);
        dbaDropDown = new JComboBox(cso_model_.getDBAs());

        dbaDropDown.insertItemAt("", 0);
        csiDropDown = new JComboBox(cso_model_.getCSIs());

	csiDropDown.insertItemAt("", 0);

        DatabaseConfig settings = cso_model_.getCurrentOverallConfig();
        notzDays.setText(Integer.toString(settings.getNotzIntervalDays()));
        notzHoursDropDown.setSelectedItem(settings.getNotzIntervalHours());
        notzMinutesDropDown.setSelectedItem(settings.getNotzIntervalMins());
        valIntervalDropDown.setSelectedItem(settings.getValidationFactor());
        algorithmDropDown.setSelectedItem(settings.getAlgorithm());


        //The DBA list for the DBA tab
        iter = cso_model_.getDBAs().iterator();
        while(iter.hasNext())
        {
            dbaListModel.addElement(iter.next());
        }
        dbaList = new JList(dbaListModel);
        //---------------------------------------




        //-------Databases panel--------
        //Databases
        dbList.setFixedCellWidth(150);
        //Allow only one value to be selected at a time
        dbList.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        //Register a selection listener
        dbList.addListSelectionListener(new ListSelectionListener() {
            //Figure out which database was selected and set the ID variable and load the DBA/CSI
            public void valueChanged(ListSelectionEvent evt) {
                // When the user release the mouse button and completes the selection,
                // getValueIsAdjusting() becomes false
                if (!evt.getValueIsAdjusting()) {
                    //Get the list
                    JList list = (JList)evt.getSource();

                    try
                    {
                        Object selectedText = list.getSelectedValue();
                        StringTokenizer tokenizer = new StringTokenizer(selectedText.toString(), " ()");
                        selectedDBName = tokenizer.nextToken();
                        selectedDBPath = tokenizer.nextToken();

                        dbaDropDown.setSelectedItem(cso_model_.getCurrentDBA(selectedDBName, selectedDBPath));
                        csiDropDown.setSelectedItem(cso_model_.getCurrentCSI(selectedDBName, selectedDBPath));
                    }
                    catch(Exception e)
                    {}
                }
            }
        });
        scrollingDBList = new JScrollPane(dbList);
        scrollingDBList.setMinimumSize(new Dimension(200, 300));
        scrollingDBList.setMaximumSize(new Dimension(200, 300));
        dbPanel1.setLayout(new BoxLayout(dbPanel1, BoxLayout.Y_AXIS));
        JLabel dbLabel = new JLabel("Databases:");
        dbPanel1.add(dbLabel);
        dbLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbPanel1.add(Box.createRigidArea(new Dimension(5, 0)));
        dbPanel1.add(scrollingDBList);
        dbPanel1.add(Box.createRigidArea(new Dimension(5, 10)));
        AddDatabaseBtn.setAlignmentX(Component.CENTER_ALIGNMENT);

        AddDatabaseBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                DatabaseDialog dbDialog = new DatabaseDialog(cso_model_);
                dbDialog.setVisible(true);
            }
        });

        dbPanel1.add(AddDatabaseBtn);

        //DBA's and CSI's
        //dropDownListener actionListener = new dropDownListener();
        //dropDown.addItemListener(actionListener);
        dbPanel2.setLayout(new BoxLayout(dbPanel2, BoxLayout.Y_AXIS));
        JLabel dbaLabel = new JLabel("DBA: ");
        dbaLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbPanel2.add(dbaLabel);
        dbaDropDown.addItem("Select Employee");
        dbaDropDown.setMaximumSize(new Dimension(300,30));
        dbaDropDown.setMinimumSize(new Dimension(300,30));
        dbPanel2.add(dbaDropDown);
        assignButtonDBA.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbPanel2.add(Box.createRigidArea(new Dimension(5, 10)));
        dbPanel2.add(assignButtonDBA);
        dbPanel2.add(Box.createRigidArea(new Dimension(5, 30)));
        JLabel csiLabel = new JLabel("CSI: ");
        csiLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbPanel2.add(csiLabel);
        csiDropDown.addItem("Select Employee");
        csiDropDown.setMaximumSize(new Dimension(300,30));
        csiDropDown.setMinimumSize(new Dimension(300,30));
        dbPanel2.add(csiDropDown);
        dbPanel2.add(Box.createRigidArea(new Dimension(5, 10)));
        assignButtonCSI.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbPanel2.add(assignButtonCSI);
        dbPanel2.add(Box.createRigidArea(new Dimension(300,30)));


        assignButtonCSI.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ae)
            {
                try
                {
                    if(csiDropDown.getSelectedIndex() != -1)
                    {
                        String csiText = csiDropDown.getSelectedItem().toString();

//                      If the DBA field is empty we need to clear the DBA for the database instead of setting a new one
                        if(csiText.equals(""))
                        {
                            cso_model_.clearCSI(selectedDBName, selectedDBPath);
                        }
                        else
                        {
                            if(selectedDBName != null)
                            {
                                String result;
                                StringTokenizer tokenizer = new StringTokenizer(csiText, "()");
                                result = cso_model_.setCurrentCSI(selectedDBName, selectedDBPath, tokenizer.nextToken(), tokenizer.nextToken());
                                if(!result.equals(""))  //There was an error
                                {
                                    JOptionPane.showMessageDialog(null, result, "Error", JOptionPane.ERROR_MESSAGE);
                                }
                                JOptionPane.showMessageDialog(null, "The CSI has been assigned.");
                                
                            }
                            else
                            {
                                JOptionPane.showMessageDialog(null, "Please select a database.", "No database selected", JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    }
                }
                catch(Exception e)
                {
                    System.err.println("Unable to parse CSI. Assignment NOT made.");
                }
            }
        });


        assignButtonDBA.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ae)
            {
                try
                {
                    if(dbaDropDown.getSelectedIndex() != -1)
                    {
                        String dbaText = dbaDropDown.getSelectedItem().toString();

                        //If the DBA field is empty we need to clear the DBA for the database instead of setting a new one
                        if(dbaText.equals(""))
                        {
                            cso_model_.clearDBA(selectedDBName, selectedDBPath);
                        }
                        else
                        {
                            if(selectedDBName != null)
                            {
                                String result;
                                StringTokenizer tokenizer = new StringTokenizer(dbaText, "()");
                                result = cso_model_.setCurrentDBA(selectedDBName, selectedDBPath, tokenizer.nextToken(), tokenizer.nextToken());
                                if(!result.equals(""))  //There was an error
                                {
                                    JOptionPane.showMessageDialog(null, result, "Error", JOptionPane.ERROR_MESSAGE);
                                }
                                    JOptionPane.showMessageDialog(null, "The DBA has been assigned");

                            }
                            else
                            {
                                JOptionPane.showMessageDialog(null, "Please select a database.", "No Database Selected", JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    }
                }
                catch(Exception e)
                {
                    System.err.println("Unable to parse DBA. Assignment NOT made.");
                }
            }
        });

        //Settings
        settingsPanel1.setLayout(new BoxLayout(settingsPanel1, BoxLayout.X_AXIS));
        JLabel algorithmLabel = new JLabel("Minimum Algorithm Choice:");
        algorithmLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel1.add(algorithmLabel);
        settingsPanel1.add(Box.createRigidArea(new Dimension(5, 0)));
        algorithmDropDown.setMaximumSize(new Dimension(150,30));
        algorithmDropDown.setMinimumSize(new Dimension(150,30));
        settingsPanel1.add(algorithmDropDown);
        settingsPanel1.setAlignmentX(Component.CENTER_ALIGNMENT);

        settingsPanel2.setLayout(new BoxLayout(settingsPanel2, BoxLayout.X_AXIS));
        JLabel notarizationLabel = new JLabel("Max Detection Resolution Unit:");
        notarizationLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel2.add(notarizationLabel);
        settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
        notzDays.setMaximumSize(new Dimension(40,30));
        notzDays.setMinimumSize(new Dimension(40,30));
        settingsPanel2.add(notzDays);
        settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel dayLabel = new JLabel("days");
        dayLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel2.add(dayLabel);
        settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
        notzHoursDropDown.setMaximumSize(new Dimension(50,30));
        notzHoursDropDown.setMinimumSize(new Dimension(50,30));
        settingsPanel2.add(notzHoursDropDown);
        settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel hourLabel = new JLabel("hrs");
        hourLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel2.add(hourLabel);
        settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
        notzMinutesDropDown.setMaximumSize(new Dimension(50,30));
        notzMinutesDropDown.setMinimumSize(new Dimension(50,30));
        settingsPanel2.add(notzMinutesDropDown);
        settingsPanel2.add(Box.createRigidArea(new Dimension(5, 0)));
        JLabel minLabel = new JLabel("mins.");
        minLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel2.add(minLabel);
        settingsPanel2.setAlignmentX(Component.CENTER_ALIGNMENT);

        settingsPanel3.setLayout(new BoxLayout(settingsPanel3, BoxLayout.X_AXIS));
        JLabel validationLabel = new JLabel("Maximum Time Between Validations:");
        validationLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel3.add(validationLabel);
        settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
        //valIntervalDropDown.setMaximumSize(new Dimension(50,30));
        //valIntervalDropDown.setMinimumSize(new Dimension(50,30));
        //settingsPanel3.add(valIntervalDropDown);
 
        notzDays1.setMaximumSize(new Dimension(40,30));
        notzDays1.setMinimumSize(new Dimension(40,30));
        settingsPanel3.add(notzDays1);
        settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
        settingsPanel3.add(new JLabel("days"));
        settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
        notzHoursDropDown1.setMaximumSize(new Dimension(50,30));
        notzHoursDropDown1.setMinimumSize(new Dimension(50,30));
        settingsPanel3.add(notzHoursDropDown1);
        settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
        settingsPanel3.add(new JLabel("hrs"));
        settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
        notzMinutesDropDown1.setMaximumSize(new Dimension(50,30));
        notzMinutesDropDown1.setMinimumSize(new Dimension(50,30));
        settingsPanel3.add(notzMinutesDropDown1);
        settingsPanel3.add(Box.createRigidArea(new Dimension(5, 0)));
        settingsPanel3.add(new JLabel("mins"));
        settingsPanel3.setAlignmentX(Component.CENTER_ALIGNMENT);

        
        settingsPanel4.setLayout(new BoxLayout(settingsPanel4, BoxLayout.X_AXIS));
        JLabel CostSingleNorLabel = new JLabel("Cost of Single Notarization:  $ ");
        CostSingleNorLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        CostSingleNorText.setText("2.25");
        CostSingleNorText.setMaximumSize(new Dimension(50,30));
        CostSingleNorText.setMinimumSize(new Dimension(50,30));
        settingsPanel4.add(CostSingleNorLabel);
        settingsPanel4.add(CostSingleNorText);
        settingsPanel4.setAlignmentX(Component.CENTER_ALIGNMENT);

        settingsPanel5.setLayout(new BoxLayout(settingsPanel5, BoxLayout.X_AXIS));
        JLabel CostSingleNorDocLabel = new JLabel("Cost to Check Single Notarized Document:    $ ");
        CostSingleNorDocLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        settingsPanel5.add(CostSingleNorDocLabel);
        CostSingleNorDocText.setText("3.25");
        CostSingleNorDocText.setMaximumSize(new Dimension(50,30));
        CostSingleNorDocText.setMinimumSize(new Dimension(50,30));
        settingsPanel5.add(CostSingleNorDocText);
        settingsPanel5.setAlignmentX(Component.CENTER_ALIGNMENT);

        dbPanel3.setLayout(new BoxLayout(dbPanel3, BoxLayout.Y_AXIS));
        dbPanel3.add(settingsPanel1);
       dbPanel3.add(Box.createRigidArea(new Dimension(5, 10)));
        dbPanel3.add(settingsPanel2);
       dbPanel3.add(Box.createRigidArea(new Dimension(5, 10)));
        dbPanel3.add(settingsPanel3);
        dbPanel3.add(Box.createRigidArea(new Dimension(5, 10)));
        dbPanel3.add(settingsPanel4);
        dbPanel3.add(Box.createRigidArea(new Dimension(5, 10)));
        dbPanel3.add(settingsPanel5);
        dbPanel3.add(Box.createRigidArea(new Dimension(5, 30)));
        saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbPanel3.add(saveButton);

        saveButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ae)
            {
                try
                {
                    //Create a new database configuration
                    DatabaseConfig config = new DatabaseConfig(
                            algorithmDropDown.getSelectedItem().toString(),
                            Integer.parseInt(notzDays.getText()),
                            Integer.parseInt(notzHoursDropDown.getSelectedItem().toString()),
                            Integer.parseInt(notzMinutesDropDown.getSelectedItem().toString()),
			    4);
//                            Integer.parseInt(valIntervalDropDown.getSelectedItem().toString()));

                    //Tell the model to save the settings
                    cso_model_.setOverallSettings(config);
                }
                catch(NumberFormatException ne)
                {
                    JOptionPane.showMessageDialog(null, "Please enter a numeric value for the notarization interval in days.", "Non-numeric value", JOptionPane.ERROR_MESSAGE);
                }
                catch(Exception e)
                {}

            }
        });


        databasePanel.setLayout(new BoxLayout(databasePanel, BoxLayout.Y_AXIS));
        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
        topPanel.add(dbPanel1);
        topPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        topPanel.add(dbPanel2);
        //databasePanel.add(dbPanel1);
        //databasePanel.add(dbPanel2);
        databasePanel.add(topPanel);
	// databasePanel.add(dbPanel3);
        //------------------------------


        //-------DBA panel--------
        //List of DBAs
        dbaList.setFixedCellWidth(150);
        //Allow only one value to be selected at a time
        dbaList.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        scrollingDBAList = new JScrollPane(dbaList);
        scrollingDBAList.setMinimumSize(new Dimension(200, 300));
        scrollingDBAList.setMaximumSize(new Dimension(200, 300));
        dbaPanelLeft.setLayout(new BoxLayout(dbaPanelLeft, BoxLayout.Y_AXIS));
        JLabel EmployeesLabel = new JLabel("Employees:");
        EmployeesLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbaPanelLeft.add(EmployeesLabel);    //!

        dbaPanelLeft.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbaPanelLeft.add(Box.createRigidArea(new Dimension(5, 0)));
        dbaPanelLeft.add(scrollingDBAList);
        dbaPanelLeft.add(Box.createRigidArea(new Dimension(5, 10)));
        AddEmployeeBtn.setAlignmentX(Component.CENTER_ALIGNMENT);

        AddEmployeeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0)
            {
                EmployeeDialog empDialog = new EmployeeDialog(cso_model_);
                empDialog.setVisible(true);
            }
        });


        dbaPanelLeft.add(AddEmployeeBtn);

        //List of managed databases
        DBAformanagedList.setFixedCellWidth(150);
        //Allow only one value to be selected at a time
        DBAformanagedList.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        CSIformanagedList.setFixedCellWidth(150);
        CSIformanagedList.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        DBAforList = new JScrollPane(DBAformanagedList);
        DBAforList.setMinimumSize(new Dimension(200, 150));
        DBAforList.setMaximumSize(new Dimension(200, 150));
        CSIforList = new JScrollPane(CSIformanagedList);
        CSIforList.setMinimumSize(new Dimension(200, 150));
        CSIforList.setMaximumSize(new Dimension(200, 150));
        dbaPanelRight.setLayout(new BoxLayout(dbaPanelRight, BoxLayout.Y_AXIS));
        JLabel DBAforLabel = new JLabel("DBA for:");
        DBAforLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        JLabel CSIforLabel = new JLabel("CSI for:");
        CSIforLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbaPanelRight.add(DBAforLabel);
        dbaPanelRight.setAlignmentX(Component.CENTER_ALIGNMENT);
        dbaPanelRight.add(Box.createRigidArea(new Dimension(5, 0)));
        dbaPanelRight.add(DBAforList);
        dbaPanelRight.add(CSIforLabel);
        dbaPanelRight.add(Box.createRigidArea(new Dimension(5, 0)));
        dbaPanelRight.add(CSIforList);

        dbaPanel.setLayout(new BoxLayout(dbaPanel, BoxLayout.X_AXIS));
        dbaPanel.add(dbaPanelLeft);
        dbaPanel.add(Box.createRigidArea(new Dimension(100, 0)));
        dbaPanel.add(dbaPanelRight);
        
	
	EmployeePanel.setLayout(new BoxLayout(EmployeePanel,BoxLayout.Y_AXIS));
	EmployeePanel.add(dbaPanel);


        //Register a selection listener for the DBA list
        dbaList.addListSelectionListener(new ListSelectionListener() {
            //Figure out which DBA was selected and get the list of databases managed
            public void valueChanged(ListSelectionEvent evt) {
                // When the user release the mouse button and completes the selection getValueIsAdjusting() becomes false
                if (!evt.getValueIsAdjusting()) {
                    //Get the list
                    //JList list = (JList)evt.getSource();

                    try
                    {
                        Object selectedText = dbaList.getSelectedValue();
                        if(selectedText != null)
                        {
                            Vector databases = cso_model_.getManagedDatabases(selectedText.toString());
                            managedListModel.clear();
                            Iterator iter = databases.iterator();
                            while(iter.hasNext())
                            {
                                managedListModel.add(managedListModel.getSize(), iter.next());
                            }
                            Vector databasesforCSI = cso_model_.getManagedDatabasesforCSI(selectedText.toString());
                            managedListModelforCSI.clear();
                            Iterator iterforCSI = databasesforCSI.iterator();
                            while(iterforCSI.hasNext())
                            {
                                managedListModelforCSI.add(managedListModelforCSI.getSize(), iterforCSI.next());
                            }
                        }
                    }
                    catch(Exception e)
                    {
                        System.err.println("Exception trying to load the databases managed.");
                        System.err.println(e);
                    }
                }
            }
        });
        //------------------------------


        //-------Problems panel--------
//      problemsListModel.add(0, "Valu Corp: Validation Failure on 1/17/07 at 02:04 AM.");
//      problemsListModel.add(1, "Johnson and Smith: Validation failure on 1/17/07 at 1:00 AM.");
//      problemsListModel.add(0, "Valu Corp: Validation Failure on 1/15/07 at 02:04 AM.");
        Iterator iter2 = cso_model_.getAllFailures().iterator();
        while(iter2.hasNext())
        {
            problemsListModel.addElement(iter2.next());
        }

        //Allow only one value to be selected at a time
	databaseDropDown.setAlignmentX(Component.LEFT_ALIGNMENT);
	databaseDropDown.setMaximumSize(new Dimension(200,30));
	databaseDropDown.setMinimumSize(new Dimension(200,30));
	//PtopPanel.add(Box.createRigidArea(new Dimension(5, 0)));        
	//PtopPanel.add(databaseDropDown);
        PtopPanel1.setLayout(new BoxLayout(PtopPanel1, BoxLayout.X_AXIS));
        JLabel PDBALabel = new JLabel("DBA: Susan Miller(sjmiller@email.arizona.edu)");
        //PDBALabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        JLabel PCSILabel = new JLabel("CSI: Dave Thompson(dt@email.arizona.edu)");
        //PCSILabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
	//PtopPanel1.add(Box.createRigidArea(new Dimension(5, 0)));        
	PtopPanel1.add(databaseDropDown);
        PtopPanel1.add(Box.createRigidArea(new Dimension(25, 5)));
        PtopPanel1.add(PDBALabel);
        //PtopPanel1.setAlignmentX(Component.LEFT_ALIGNMENT);
	//PtopPanel1.add(Box.createRigidArea(new Dimension(5, 0)));        
	//PtopPanel1.add(PCSILabel);

        PtopPanel2.setLayout(new BoxLayout(PtopPanel2, BoxLayout.X_AXIS));
        PtopPanel2.add(Box.createRigidArea(new Dimension(220, 5)));
        PtopPanel2.add(PCSILabel);
        //PtopPanel2.setAlignmentX(Component.RIGHT_ALIGNMENT);

        dbList.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        scrollingProblemsList = new JScrollPane(problemsList);
        problemsPanel.setLayout(new BoxLayout(problemsPanel, BoxLayout.Y_AXIS));
        JLabel problemsLabel = new JLabel("Detected Tampering:");
        problemsLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        //problemsPanel.add(databaseDropDown);
        problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        problemsPanel.add(PtopPanel1);
        problemsPanel.add(PtopPanel2);
        problemsPanel.add(problemsLabel);
        //problemsLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        problemsPanel.add(scrollingProblemsList);
        problemsPanel.add(Box.createRigidArea(new Dimension(25, 5)));

        JLabel FACostSingleTamperLabel = new JLabel("Forensic analysis cost to analyze this tampering is  $ ");
	FACostSingleTamperLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        FAcostPanel1.setLayout(new BoxLayout(FAcostPanel1, BoxLayout.X_AXIS));
        FAcostPanel1.add(FACostSingleTamperLabel);
        FAcostText1.setText("1.25");
        FAcostText1.setMaximumSize(new Dimension(50,30));
        FAcostText1.setMinimumSize(new Dimension(50,30));
        FAcostPanel1.add(FAcostText1);
	FAcostPanel1.setAlignmentX(Component.CENTER_ALIGNMENT);


	problemsPanel.add(FAcostPanel1);
        JLabel FACostSingleDBLabel = new JLabel("Forensic analysis cost to analyze this database is  $ ");
	//FACostSingleDBLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        FAcostPanel2.setLayout(new BoxLayout(FAcostPanel2, BoxLayout.X_AXIS));
        FAcostPanel2.add(FACostSingleDBLabel);
        FAcostText2.setText("2.25");
        FAcostText2.setMaximumSize(new Dimension(50,30));
        FAcostText2.setMinimumSize(new Dimension(50,30));
        FAcostPanel2.add(FAcostText2);
	FAcostPanel2.setAlignmentX(Component.CENTER_ALIGNMENT);

	problemsPanel.add(FAcostPanel2);
        JLabel FACostAllDBLabel = new JLabel("Forensic analysis cost to analyze all my databases is  $ ");
	//FACostAllDBLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        FAcostPanel3.setLayout(new BoxLayout(FAcostPanel3, BoxLayout.X_AXIS));
        FAcostPanel3.add(FACostAllDBLabel);
        FAcostText3.setText("3.25");
        FAcostText3.setMaximumSize(new Dimension(50,30));
        FAcostText3.setMinimumSize(new Dimension(50,30));
        FAcostPanel3.add(FAcostText3);
	FAcostPanel3.setAlignmentX(Component.CENTER_ALIGNMENT);

	problemsPanel.add(FAcostPanel3);
        problemsPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        viewBtn.setAlignmentX(Component.CENTER_ALIGNMENT);
        problemsPanel.add(Box.createRigidArea(new Dimension(25, 5)));
        problemsPanel.add(viewBtn);
        problemsPanel.add(Box.createRigidArea(new Dimension(25, 5)));
        problemsPanel.setAlignmentX(Component.CENTER_ALIGNMENT);


        //------------------------------

        pane.addTab("Employees", EmployeePanel);
        pane.addTab("Global Settings",dbPanel3);
        pane.addTab("Databases", databasePanel); 
        pane.addTab("Detected Tampering", problemsPanel);
        frame.getContentPane().add(pane);
        frame.setSize(500, 500);
        frame.setVisible(true);
	frame.setLocation(screenWt/3,screenHt/3);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }



    /* 
     * Refreshes all the lists and drop down boxes in the application and
     * selects the current values.
     * 
     * (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */


	private void loadDatabases()
	{

                databaseDropDown.addItem("All Databases");

	}



    public void update(Observable arg0, Object arg1)
    {    
        String next;

        //DBA drop-down and DBA list (on DBA tab)
        dbaDropDown.removeAllItems();
        dbaListModel.clear();
        Iterator iter = cso_model_.getDBAs().iterator();

        while(iter.hasNext())
        {
            next = iter.next().toString();
            dbaDropDown.addItem(next);
            dbaListModel.addElement(next);
        }
        dbaDropDown.insertItemAt("", 0);
        dbaDropDown.validate();
        scrollingDBAList.validate();

        //CSI drop-down
        csiDropDown.removeAllItems();
        iter = cso_model_.getCSIs().iterator();

        while(iter.hasNext())
        {
            csiDropDown.addItem(iter.next());
        }
        csiDropDown.insertItemAt("", 0);
        csiDropDown.validate();

        //Database list
        listModel.clear();
        iter = cso_model_.getDatabases().iterator();
        while(iter.hasNext())
        {
            listModel.addElement(iter.next());
        }
        scrollingDBList.validate();
        dbPanel1.validate();
        
        //Tamper detections
        problemsListModel.clear();
        iter = cso_model_.getAllFailures().iterator();
        while(iter.hasNext())
        {
            problemsListModel.addElement(iter.next());
        }
        scrollingProblemsList.validate();
    }

}
