package gui;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.*;

/**
 * Displays the login screen for the Database Monitor Application.
 * If the employee can be verified in the Security Master Database
 *  as a valid employee the application is then launched.
 * 
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
class DBALoginDialog extends JDialog implements ActionListener
{
    //Turn on for debugging
    private boolean debug = false;

    private JTextField email;
    private JPasswordField password;
    Toolkit kit = Toolkit.getDefaultToolkit();
    Dimension screenSize = kit.getScreenSize();
    int screenHt = screenSize.height;
    int screenWt = screenSize.width;
	static Boolean bUseUnits = false;	//Default to NOT showing values in Units    

    
    private String notarizer_server_;
    private String binary_dir_;
    private String dns_server_;
    private String surety_password_;
    private String surety_account_name_;
    private int mode_;

    
    /**
     * Sets up the GUI components for the dialog box
     */
    public DBALoginDialog()
    {
        JLabel emailLabel = new JLabel("Email:", JLabel.TRAILING);
        email = new JTextField(50);
        JLabel passwordLabel = new JLabel("Password:", JLabel.TRAILING);
        password = new JPasswordField(50);
        JButton loginButton = new JButton("Login");
        loginButton.addActionListener(this);
        JButton exitButton = new JButton("Exit");
        exitButton.addActionListener(this);

        setTitle("Login");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);


        //Create and populate panel
        JPanel topPanel = new JPanel(new SpringLayout());
        topPanel.add(emailLabel);
        topPanel.add(email);
        topPanel.add(passwordLabel);
        topPanel.add(password);
        topPanel.add(exitButton);
        topPanel.add(loginButton);

//      Lay out the panel.
        SpringUtilities.makeCompactGrid(topPanel,
                3, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad


        //Add the panel to this JDialog
        setContentPane(topPanel);
        setSize(screenWt/4,screenHt/4);
	setLocation(screenWt/4,screenHt/3);
        pack();
    }


    /* 
     * Responds to the Login and Exit buttons.
     * On "Exit" it simply exits the program.
     * On "Login" then the employee is checked in the database and is valid,
     * the application is then launched. Otherwise an appropriate error
     * message is displayed to the user.
     * 
     * (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent ae)
    {
        if(ae.getActionCommand().equals("Login"))
        {
            //Check the database to see if this login is valid
            Connection conn = Connector.getConnection();
            String query = "select Name from Employee where Email = '" + email.getText() + "' and Password='" + String.valueOf(password.getPassword()) + "' and isDBA='Y';";
            if(debug) { System.out.println(query); };
            if(conn != null)
            {
                try
                {
                    Statement stmt = conn.createStatement();
                    ResultSet rset = stmt.executeQuery(query);

                    if(rset.next()) //This login is valid
                    {
/* Modified by Huilong on 5/14/2007 */
                        String name=rset.getString("Name");
                        /* Michael Patterson 10/8/2008 - Added Units boolean to CSIView */
                        //new DBAView(name, email.getText());
                        new DBAView(name, email.getText(), bUseUnits,
                                    mode_,
                                    notarizer_server_, binary_dir_,
                                    dns_server_,
                                    surety_password_, surety_account_name_);
                        /* MP - End Units boolean change */
/* End Huilong */
                        //Close the login dialog
                        this.setVisible(false);
                        this.dispose();
                    }
                    else    //Invalid login
                    {
                        JOptionPane.showMessageDialog(null, "The email and password did not match or the user is not designated as a DBA.", "Invalid Login", JOptionPane.ERROR_MESSAGE);
                        return;
                    }

                }
                catch (SQLException se)
                {
                    System.err.println("SQLException while attempting to login.");
                    System.err.println(se.toString());
                }
                catch(Exception e)
                {
                    System.err.println("Exception while attempting to login.");
                    System.err.println(e);
                }
            }
        }
        else if(ae.getActionCommand().equals("Exit"))
        {
            System.exit(0);
        }

    }
    
    public void setDNSServer(String dns_server) {
      dns_server_ = dns_server;
    }

    public void setNotarizerServer(String notarizer_server) {
      notarizer_server_ = notarizer_server;
    }
        
    public void setBinaryDir(String binary_dir) {
      binary_dir_ = binary_dir;    	
    }
    
    public void setSuretyPassword(String password) {
      surety_password_ = password;
    }
    
    public void setSuretyAccountName(String account_name) {
      surety_account_name_ = account_name;
    }
    
    public void setMode(int mode) {
      mode_ = mode;
    }

    /**
     * Invokes the Database Monitor Application
     */
    public static void main(String[] args) {
    	/* Michael Patterson 10/8/2008 - Added option for displaying Units
    	 * instead of dollars.
    	 */
    	if(args.length > 0 && args[0].equalsIgnoreCase("units"))
    	{
    		bUseUnits = true;
    	}
    	/* MP - End Units option */    	
        
        
        if (args.length != 6 && args.length != 8) {
    		Usage();
    		return;
    	}
        String ns = null;
        String dir = null;
        String dns = null;
        String account = null;
        String password = null;
        int matched_args = 0;
        for (int i = 0; i < args.length; ++i) {
          if (args[i].equals("-NS")) {
            ++i;
            ns = args[i];
            ++matched_args;
          } else if (args[i].equals("-DIR")) {
            ++i;
            dir = args[i];
            ++matched_args;
          } else if (args[i].equals("-DNS")) {
            ++i;
            dns = args[i];
            ++matched_args;
          } else if (args[i].equals("-A")) {
            ++i;
            account = args[i];
            ++matched_args;
          } else if (args[i].equals("-P")) {
            ++i;
            password = args[i];
            ++matched_args;
          }
        }
        if (matched_args != 3 && matched_args != 4) {
          Usage();
          return;
        }
        	
        DBALoginDialog dialog = new DBALoginDialog();
        dialog.setNotarizerServer(ns);
        dialog.setBinaryDir(dir);
        if (matched_args == 3) {
          dialog.setMode(0);
          dialog.setDNSServer(dns);
        } else if (matched_args == 4) {
          dialog.setMode(1);
          dialog.setSuretyPassword(password);
          dialog.setSuretyAccountName(account);
        }
        dialog.setVisible(true);
    }
    
    public static void Usage() {    	
      System.out.println(
          "Usage:\n  java -jar DBA.jar -NS notarizer_server -DIR binary_dir -DNS dns_server");
      System.out.println("    OR");
      System.out.println(
              "  java -jar DBA.jar -NS notarizer_server -DIR binary_dir " +
              "-A surety_username -P surety_password");
      System.out.println("Options:");
      System.out.println("  -NS: the name of the machine where the Secure Master Database is installed");
      System.out.println("  -DIR: the absolute path name of the directory where both the validator and notarizer exist");
      System.out.println("  -DNS: the name of the machine where the local DNS service runs");
      System.out.println("  -A: the account name for accessing Surety service");
      System.out.println("  -P: the password for accessing Surety service");
    }
}
