package gui;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

/**
 * The dialog used to add a database to the Security Master database.
 * 
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
class DatabaseDialog extends JDialog implements ActionListener
{
    private CSOModel model;
    private JTextField name;
    private JTextField source;


    /**
     * Setups the GUI components needed to display this dialog.
     * 
     * @param the CSOModel to use for adding this database
     */
    public DatabaseDialog(CSOModel inModel)
    {
        //Always a good idea to call super
        super();
        this.model = inModel;

        JLabel dbNameLabel = new JLabel("Database Name:", JLabel.TRAILING);
        name = new JTextField(50);
        JLabel dbSourceLabel =
        	new JLabel("Database Source: (This must be an absolute path)",
                       JLabel.TRAILING);
        source = new JTextField(50);
        JButton saveButton = new JButton("Save");
        saveButton.addActionListener(this);

        setTitle("Add Database");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);


        //Create and populate panel
        JPanel topPanel = new JPanel(new SpringLayout());
        topPanel.add(dbNameLabel);
        topPanel.add(name);
        topPanel.add(dbSourceLabel);
        topPanel.add(source);
        topPanel.add(new JLabel(""));
        topPanel.add(saveButton);

//      Lay out the panel.
        SpringUtilities.makeCompactGrid(topPanel,
                3, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad


        //Add the panel to this JDialog
        setContentPane(topPanel);
        pack();
    }


    /* 
     * Responds to the "Save" button.
     * If all the fields contain data, the input is saved to the database and the dialog is closed.
     * If fields are left blank or the model returns an error a message is displayed to the user
     * and the dialog is not closed.
     * 
     * (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent ae)
    {
        if(ae.getActionCommand().equals("Save"))
        {
            if(model == null)
            {
                System.err.println("Model is null, cannot save database");
                return;
            }
            else
            {
                //Check to make sure that input has been entered
                if(name.getText().equals("") || source.getText().equals(""))
                {
                    JOptionPane.showMessageDialog(null, "Please fill in all fields!", "Empty field(s)", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                String tmp_name = name.getText();
                if (tmp_name.length() > 512) {
                	JOptionPane.showMessageDialog(null, "Database Name too long", "Error!", JOptionPane.ERROR_MESSAGE);
                	return;
                }
                String tmp_path = source.getText();
                if (tmp_path.length() > 1024) {
                	JOptionPane.showMessageDialog(null, "Database Path too long", "Error!", JOptionPane.ERROR_MESSAGE);
                	return;
                }
                String result = model.addDatabase(name.getText(), source.getText());

                if(result.equals(""))
                {   //Success
                    model.notifyObservers();

                    //Close the dialog
                    this.setVisible(false);
                    this.dispose();
                }
                else
                {
                    JOptionPane.showMessageDialog(null, result, "Error!", JOptionPane.ERROR_MESSAGE);

                }
            }
        }

    }

}