package gui;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

/**
 * The dialog used to add an employee to the Security Master database.
 * 
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
class EmployeeDialog extends JDialog implements ActionListener
{
    private CSOModel model;
    private JTextField name;
    private JTextField email;
    private JTextField password;
    private JTextField phone;
    private JCheckBox isCSI;
    private JCheckBox isDBA;


    /**
     * Setups the GUI components needed to display this dialog.
     * 
     * @param the CSOModel to use for adding this employee
     */
    public EmployeeDialog(CSOModel inModel)
    {
        //Always a good idea to call super
        super();
        this.model = inModel;

        JLabel nameLabel = new JLabel("Name:", JLabel.TRAILING);
        name = new JTextField(35);
        JLabel emailLabel = new JLabel("Email:", JLabel.TRAILING);
        email = new JTextField(35);
        JLabel passwordLabel = new JLabel("Password:", JLabel.TRAILING);
        password = new JTextField(10);
        JLabel phoneLabel = new JLabel("Phone:", JLabel.TRAILING);
        phone = new JTextField(35);
        isCSI = new JCheckBox("CSI");
        isDBA = new JCheckBox("DBA");
        JButton saveButton = new JButton("Save");
        saveButton.addActionListener(this);

        setTitle("Add Employee");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);


        //Create and populate panel
        JPanel topPanel = new JPanel(new SpringLayout());
        topPanel.add(nameLabel);
        topPanel.add(name);
        topPanel.add(emailLabel);
        topPanel.add(email);
        topPanel.add(passwordLabel);
        topPanel.add(password);
        topPanel.add(phoneLabel);
        topPanel.add(phone);
        topPanel.add(new JLabel(""));
        topPanel.add(isCSI);
        topPanel.add(new JLabel(""));
        topPanel.add(isDBA);
        topPanel.add(new JLabel(""));
        topPanel.add(saveButton);

//      Lay out the panel.
        SpringUtilities.makeCompactGrid(topPanel,
                7, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad



        //Add the panel to this JDialog
        setContentPane(topPanel);
        pack();
    }


    /* 
     * Responds to the "Save" button.
     * If all the fields contain data, the input is saved to the database and the dialog is closed.
     * If fields are left blank or the model returns an error a message is displayed to the user
     * and the dialog is not closed.
     * 
     * (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent ae)
    {
        if(ae.getActionCommand().equals("Save"))
        {
            if(model == null)
            {
                System.err.println("Model is null, cannot save database");
                return;
            }
            else
            {
                //Check to make sure that input has been entered
                if(name.getText().equals("") || email.getText().equals("") 
                        || password.getText().equals("")
                        || phone.getText().equals(""))
                {
                    JOptionPane.showMessageDialog(null, "Please fill in all fields!", "Empty field(s)", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                String result = model.addEmployee(name.getText(), email.getText(), isCSI.isSelected(), isDBA.isSelected(), password.getText(), phone.getText());
                if(result.equals(""))
                {   //Success
                    model.notifyObservers();

                    //Close the dialog
                    this.setVisible(false);
                    this.dispose();
                }
                else
                {
                    JOptionPane.showMessageDialog(null, result, "Error!", JOptionPane.ERROR_MESSAGE);

                }
            }
        }

    }

}