#include <sys/types.h>
#include <stdio.h>
#include <time.h>
#include <sys/times.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <errno.h>
#include <pthread.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "db.h"

# define MAX_NUM_RECORDS 1000
# define FALSE 0
# define TRUE 1
# define NUM_WORKERS 2

char ENV_DIRECTORY[256];
const char prefixMsg[] = "Application";
char dbname[256];
char dbpathname[256];

int simpleTest(char * dbName, char * dbNameWithEnv, int openFlag);
int dbStartup(char *dbName, DB **dbpPtr, DB_ENV **dbenvPtr, DB_TXN ** tidPtr, int openFlag);
int dbShutdown(DB *dbp, DB_ENV *dbenv, DB_TXN * tid, int printTT);
int insertData(DB *dbp, DB_TXN *tid, int keyVal, char * strData);

int main (int argc, char **argv) {

    int retVal, numRecords, numVersions, numRecordsToRead,
        openFlag = DB_CREATE|DB_TEMPORAL|DB_AUDIT;
        //openFlag = DB_CREATE|DB_TEMPORAL;
    char command[256];

    if(argc!=3){
        printf("Usage: %s db_path db_name\n", argv[0]);
        exit(1);
    }

    ENV_DIRECTORY[0]='\0';
    sprintf(ENV_DIRECTORY, "%s", argv[1]);
    dbname[0]='\0';
    strcpy(dbname, argv[2]);
    sprintf(dbpathname, "%s/%s", ENV_DIRECTORY, dbname);

    command[0]='\0';
    sprintf(command, "rm -rf %s", ENV_DIRECTORY);
    system(command);
    command[0]='\0';
    sprintf(command, "mkdir %s", ENV_DIRECTORY);
    system(command);
    command[0]='\0';
    sprintf(command, "chmod go+rw %s", ENV_DIRECTORY);
    system(command);

    /* Test 1 */
    printf("[%s]Test 1 --@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n", prefixMsg);
    retVal = simpleTest(dbname, dbpathname, openFlag);

    return 0;
}

/* Added by Huilong Huang
 */
int my_pause(int sec, int msec)
{
        int select_result;
        fd_set read_fds;
        struct timeval timeout;
        unsigned char c[256];

        FD_ZERO( &read_fds );
        FD_SET( STDIN_FILENO, &read_fds );

        timeout.tv_sec = sec;
        timeout.tv_usec = 0;
        select_result = select( 1, &read_fds, NULL, NULL, &timeout );

        if(select_result>0){
            read(STDIN_FILENO, &c, 128);
//            if(c == 'k')
             return 1;
/*            FD_ZERO( &read_fds );
            FD_SET( STDIN_FILENO, &read_fds );
            select(1, &read_fds, NULL, NULL, NULL);
            read(STDIN_FILENO, &c, 128);*/
        }
        return 0;

/*        FD_ZERO( &read_fds );
        FD_SET( STDIN_FILENO, &read_fds );

        timeout.tv_sec = sec;
        timeout.tv_usec = msec*1000;
        select_result = select( 1, &read_fds, NULL, NULL, &timeout );

        if(select_result>0){
           read(STDIN_FILENO, &c, 256);
        }
        getchar();
        return;*/
}
/* End Huilong */

int simpleTest(char * dbName, char * dbNameWithEnv, int openFlag)
{
    DB *dbp;
    DBC *dbcp;
    DBT key, data;
    DB_ENV *dbenv;
    DB_TXN * tid;
    int i,ret, keyVal;
    struct timeval startupTime, shutdownTime;
    char strData[100], strData2[100], strData3[100];
    char command[128];
char hash_value[AUD_HASH_LEN];
struct timeval ttime, current_t;

    /**************** DATA INSERTION PHASE *****************/
    printf("[%s] PASS-1: Insert data into the database\n", prefixMsg);

    if ((ret = dbStartup(dbName, &dbp, &dbenv, &tid, openFlag)) != 0) {
        printf("Error in dbStartup\n");
        return ret;
    }

    command[0]='\0';
    sprintf(command, "chmod g+rw %s/*", ENV_DIRECTORY);
    system(command);

    //for (keyVal = 0; keyVal < 10; keyVal ++) {
    keyVal = 0;
    while(1){
//        for(i=0;i<10;i++){
            keyVal++;
            if ((ret = txn_begin(dbenv, NULL, &tid, 0)) != 0) {
                dbenv->err(dbenv, ret, "DB_ENV->txn_begin");
                return (-1);
            }
            //for (i = 0; i < rand() % 5 + 1; ++i) {
//if(keyVal<136){
            
            sprintf(strData, "Data item %d\n", keyVal);
            insertData(dbp, tid, keyVal, strData);
/*            
            sprintf(strData2, "Data item %d", keyVal+1000000);
            insertData(dbp, tid, keyVal+1000000, strData2);
            if (keyVal %2 == 1) {
             sprintf(strData2, "Data item %d", keyVal+5000000);
             insertData(dbp, tid, keyVal+500000, strData3);
             
            }
*/
//}
//printf("Press enter to continue ...");
//getchar();
printf("Before transaction commit!\n");
            if ((ret = txn_commit(tid,0)) != 0) {
                dbenv->err(dbenv, ret, "DB_TXN->commit");
                return (-1);
            }
printf("After transaction commit!\n");
/* Changed by Huilong
 */
//__aud_get_nrequest(dbenv, hash_value, &ttime);
//printf("Transaction time(%d, %d), hash=", ttime.tv_sec, ttime.tv_usec);print_hash(hash_value);
//        }
//        sleep(10);
        dbp->db->sync(dbp->db, 0);
        if(my_pause(5,0)) break;
/* End Huilong */
    }

 if ((ret = dbShutdown(dbp, dbenv, tid, FALSE)) != 0)
        return ret;

//for debug only
/*   struct timeval start,stop,ret_start,ret_stop;
   int counter = 0;

    start.tv_sec = 0;
    start.tv_usec = 0;
    stop.tv_sec = FOREVER;
    stop.tv_usec = 0;

    if ((ret = dbStartup(dbName, &dbp, &dbenv, &tid, openFlag)) != 0) {
        printf("Error in dbStartup\n");
        return ret;
    }

    //Begin the transaction.
    if ((ret = txn_begin(dbenv, NULL, &tid, 0)) != 0) {
        dbenv->err(dbenv, ret, "DB_ENV->txn_begin");
        return (-1);
    }

    printf("Reader : Transaction Started \n");

    // Acquire a cursor for the database.
    if ((ret = dbp->cursor(dbp, tid, &dbcp, 0)) != 0) {
        dbp->err(dbp, ret, "DB->cursor");
        return(-1);
    }


    // Initialize the key/data return pair.
    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));

   while ((ret = (dbcp->c_get_nonsequenced(dbcp, &key, &data,
                   start, stop, DB_NEXT, &ret_start, &ret_stop))) == 0)
//    while ((ret = dbcp->c_get(dbcp, &key, &data, DB_NEXT)) == 0)

    {
        printf("    c_get_nonsequenced(): Found a tuple: %s\n", (char *)data.data);
printf("Start time:<%d,%d>", ret_start.tv_sec, ret_start.tv_usec);
printf("Stop time:<%d,%d>", ret_stop.tv_sec, ret_stop.tv_usec);
        counter++;
        sleep(1);
    }
printf("ret is %d, total tuples in DB is : %d\n",ret,counter);

    if ((ret = dbcp->c_close(dbcp)) != 0) {
        dbp->err(dbp, ret, "db cursor close");
    }

    if (ret != DB_NOTFOUND && ret != 0) {
        dbp->err(dbp, ret, "DBcursor->get");
        return(-1);
    }


     if ((ret = txn_commit(tid,0)) != 0) {
        dbenv->err(dbenv, ret, "DB_TXN->commit");
        return (-1);
    }
//*/
    return 0;
}


int dbStartup(char *dbName, DB **dbpPtr, DB_ENV **dbenvPtr, DB_TXN ** tidPtr, int openFlag) {

    int ret;
    DB_ENV *dbenv;
    DB * dbp;

    printf("enter dbStartup\n");
    /*Create the environment handle. */
    if ((ret = db_env_create(dbenvPtr, 0)) != 0) {
        printf("[%s] dbStartup: db_env_create: %s\n", prefixMsg, db_strerror(ret));
        return (-1);
    }

    dbenv = *dbenvPtr;
    /* Opening the Environment and setting its options */
    if ((ret = dbenv->open(dbenv, ENV_DIRECTORY,
                           DB_CREATE | DB_INIT_LOCK | DB_INIT_LOG |
                           DB_INIT_MPOOL | DB_INIT_TXN | DB_RECOVER | DB_THREAD,
                           0664)) != 0) {
        printf("[%s] dbStartup: DB_ENV->open: %s\n", prefixMsg, db_strerror(ret));
        (void)dbenv->close(dbenv, 0);
        dbenv->err(dbenv, ret, "dbenv->open: %s", ENV_DIRECTORY);
        return(-1);
    }


   // Creating a db handle under the dbenvironment
    if ((ret = db_create(dbpPtr, dbenv, 0)) != 0) {
        printf("[%s] dbStartup: db_create: %s\n", prefixMsg, db_strerror(ret));
        return(-1);
    }

    dbp = *dbpPtr;

    if ((ret = dbp->open(dbp, dbName, NULL, DB_BTREE, openFlag, 0664)) != 0) {
        printf("[%s] dbStartup: DB->open: %s\n", prefixMsg, db_strerror(ret));
        dbp->err(dbp, ret, "db->open: %s", dbName);
        return(-1);
    }

    printf("[%s] DATABASE OPENED\n", prefixMsg);

    return (0);
}

int dbShutdown(DB *dbp, DB_ENV *dbenv, DB_TXN * tid, int printTT) {

    int ret;

    /*
    if (tid != NULL) {
        if ((ret = txn_commit(tid,0)) != 0) {
            dbenv->err(dbenv, ret, "DB_TXN->commit");
            return (-1);
        }
    }
    */

    if ((ret = dbp->close(dbp, 0)) != 0) {
        dbp->err(dbp, ret, "db close");
        return(-1);
    }
/*
    if ((ret = dbenv->close(dbenv,0)) != 0) {
        dbenv->err(dbenv, ret, "db env close");
        return(-1);
    }
*/
//    sleep(2);
    printf("[%s] DATABASE CLOSED \n", prefixMsg);
    return (0);
}

int insertData(DB *dbp, DB_TXN *tid, int keyVal, char * strData) {

    DBT key, data;
    int ret;
    struct timeval tv;
    char timestring[256];
    char millisec[256];
    char *format="";

    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
    key.data = (int *) &keyVal;
    key.size = sizeof(keyVal);
    data.data = strData;
    data.size = strlen(strData)+1;
    gettimeofday(&tv, NULL);
    millisec[0]='\0';
    sprintf(millisec, "(%d:%d)", tv.tv_sec, tv.tv_usec);
    strftime(timestring, 256, "%a, %d %b %Y %H:%M:%S:", localtime(&(tv.tv_sec)));
    strcat(timestring, millisec);
    if ((ret = dbp->put(dbp, tid, &key, &data, 0)) == 0)
        printf("[%s: %s] (key, data) stored: (%d, %s) \n", prefixMsg, timestring, *(int *)key.data, (char *)data.data);
    else {
        printf("[%s] ERROR in insertData()\n", prefixMsg);
        /*dbp->err(dbp, ret, "DB->put");*/
        /* Modify !!!!!!!!!!!!!!!!!!!!!!!*/
        exit(-1);
    }

    return(0);
}


