#include <sys/types.h>
#include <stdio.h>
#include <time.h>
#include <sys/times.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <errno.h>
#include <pthread.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "db.h"

char ENV_DIRECTORY[256];
const char prefixMsg[] = "Application";
char dbname[256];
char dbpathname[256];

int detect_creation_time(char * dbName, char * dbNameWithEnv, int openFlag);
int dbStartup(char *dbName, DB **dbpPtr, DB_ENV **dbenvPtr, DB_TXN ** tidPtr, int openFlag);
int dbShutdown(DB *dbp, DB_ENV *dbenv, DB_TXN * tid, int printTT);

int main (int argc, char **argv) {
  int retVal, numRecords, numVersions, numRecordsToRead,
      openFlag = DB_TEMPORAL|DB_AUDIT;
  int result;
  struct stat buf;

  if(argc!=3){
    printf("Usage: %s db_path db_name\n", argv[0]);
    printf("Note: db_path must be an absolute path string.\n");
    exit(1);
  }
    ENV_DIRECTORY[0]='\0';
    sprintf(ENV_DIRECTORY, "%s", argv[1]);
    dbname[0]='\0';
    strcpy(dbname, argv[2]);
    sprintf(dbpathname, "%s/%s", ENV_DIRECTORY, dbname);
        
    if(stat(dbpathname, &buf)!=0){
       printf("Data base does not exist!\n");
       exit(1);
    }
    result=detect_creation_time(dbname, dbpathname, openFlag);

    return 0;
}

int detect_creation_time(char * dbName, char * dbNameWithEnv, int openFlag)
{
    DB *dbp;
    DBC *dbcp;
    DBT key, data;
    DB_ENV *dbenv;
    DB_TXN * tid;
    int i,ret, keyVal;
    struct timeval startupTime, shutdownTime;
    char strData[100], strData2[100];
    char time_str[128];
   struct timeval start,stop,ret_start,ret_stop, db_ctime;
   int counter = 0;

    start.tv_sec = BEGINNING;
    start.tv_usec = 0;
    stop.tv_sec = FOREVER;
    stop.tv_usec = 0;
    db_ctime.tv_sec = FOREVER;
    db_ctime.tv_usec = 0;
    if ((ret = dbStartup(dbName, &dbp, &dbenv, &tid, openFlag)) != 0) {  
//        printf("Error in dbStartup\n");
        return ret;
    }

    //Begin the transaction.
    if ((ret = txn_begin(dbenv, NULL, &tid, 0)) != 0) {
        dbenv->err(dbenv, ret, "DB_ENV->txn_begin");
        return (-1);
    }

    // Acquire a cursor for the database.
    if ((ret = dbp->cursor(dbp, tid, &dbcp, 0)) != 0) {
        dbp->err(dbp, ret, "DB->cursor");
        return(-1);
    }


    // Initialize the key/data return pair.
    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));

    /* Walk through the database and print out the key/data pairs. */
    while ((ret = dbcp->c_get_nonsequenced(dbcp, &key, &data, start, stop, DB_NEXT, &ret_start, &ret_stop)) == 0) {
        if(ret_start.tv_sec < db_ctime.tv_sec ||
          (ret_start.tv_sec == db_ctime.tv_sec && ret_start.tv_usec<db_ctime.tv_usec))
         db_ctime = ret_start;
    }


    if(db_ctime.tv_sec!=FOREVER){
      strftime(time_str,30,"%Y-%m-%d %X",localtime(&(db_ctime.tv_sec)));
      printf("Start time: %s\n", time_str);
    }
    else{
        //printf("ret is %d, total tuples in DB is : %d\n",ret,counter);
       printf("Database is empty!\n");
    }

    if ((ret = dbcp->c_close(dbcp)) != 0) {
        dbp->err(dbp, ret, "db cursor close");
    }

    if (ret != DB_NOTFOUND && ret != 0) {
        dbp->err(dbp, ret, "DBcursor->get");
        return(-1);
    }


     if ((ret = txn_commit(tid,0)) != 0) {
        dbenv->err(dbenv, ret, "DB_TXN->commit");
        return (-1);
    }
    return 0;
}


int dbStartup(char *dbName, DB **dbpPtr, DB_ENV **dbenvPtr, DB_TXN ** tidPtr, int openFlag) {

    int ret;
    DB_ENV *dbenv;
    DB * dbp;

//   printf("enter dbStartup\n");
    /*Create the environment handle. */
    if ((ret = db_env_create(dbenvPtr, 0)) != 0) {
//        printf("[%s] dbStartup: db_env_create: %s\n", prefixMsg, db_strerror(ret));
        return (-1);
    }

    dbenv = *dbenvPtr;
    /* Opening the Environment and setting its options */
    if ((ret = dbenv->open(dbenv, ENV_DIRECTORY,
                           DB_JOINENV | DB_THREAD,
                           0664)) != 0) {
  //      printf("[%s] dbStartup: DB_ENV->open: %s\n", prefixMsg, db_strerror(ret));
        (void)dbenv->close(dbenv, 0);
        dbenv->err(dbenv, ret, "dbenv->open: %s", ENV_DIRECTORY);
        return(-1);
    }


   // Creating a db handle under the dbenvironment
    if ((ret = db_create(dbpPtr, dbenv, 0)) != 0) {
//        printf("[%s] dbStartup: db_create: %s\n", prefixMsg, db_strerror(ret));
        return(-1);
    }

    dbp = *dbpPtr;

    if ((ret = dbp->open(dbp, dbName, NULL, DB_BTREE, openFlag, 0664)) != 0) {
//        printf("[%s] dbStartup: DB->open: %s\n", prefixMsg, db_strerror(ret));
        dbp->err(dbp, ret, "db->open: %s", dbName);
        return(-1);
    }

//    printf("[%s] DATABASE OPENED\n", prefixMsg);

    return (0);
}

int dbShutdown(DB *dbp, DB_ENV *dbenv, DB_TXN * tid, int printTT) {

    int ret;

    /*
    if (tid != NULL) {
        if ((ret = txn_commit(tid,0)) != 0) {
            dbenv->err(dbenv, ret, "DB_TXN->commit");
            return (-1);
        }
    }
    */

    if ((ret = dbp->close(dbp, 0)) != 0) {
        dbp->err(dbp, ret, "db close");
        return(-1);
    }

    if ((ret = dbenv->close(dbenv,0)) != 0) {
        dbenv->err(dbenv, ret, "db env close");
        return(-1);
    }

    return (0);
}
