#include "dnsverifier.h"

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/times.h>

#include "sha1.h"
#include "common.h"
//#include "validator.h"

#define TMP "/tmp"
#define SURETY_BINARY "testnc"

typedef struct sockaddr SA;
typedef struct sockaddr_in SAI;

void CommunicateWithLocalDNS(VerificationParameter* ver_param) {
  SAI servaddr;
  in_port_t port;
  port = 11111;  //for test only
  struct hostent *serv_p;
  int sockfd;
  int n;

  memset(&servaddr, 0, sizeof(servaddr));
  servaddr.sin_family = AF_INET;
  servaddr.sin_port = htons(port);
  serv_p = gethostbyname(ver_param->server_name);
  bcopy(serv_p->h_addr, &servaddr.sin_addr, serv_p->h_length);
  sockfd = socket(AF_INET, SOCK_DGRAM, 0);

  sendto(sockfd, ver_param->hash_value, AUD_HASH_LEN, 0,
         (SA *)&servaddr, sizeof(servaddr));

  printf("Request sent: ttime=(%d, %d)\n",
         ver_param->ttime.tv_sec, ver_param->ttime.tv_usec);
  printf("              thash=");
  print_hash(ver_param->hash_value);

  n = recvfrom(sockfd, &(ver_param->nid_output),
               sizeof(ver_param->nid_output), 0, NULL, NULL);

  printf("    N-id received: n=%d, ntime=(%d, %d)\n",
         ver_param->nid_output.ntime.tv_sec,
         ver_param->nid_output.ntime.tv_usec);
  printf("!!!!Nid recalculated:");
  print_hash(ver_param->nid_output.nid);
}


void CommunicateWithSuretyDNS(VerificationParameter* ver_param) {
  char notarizedFileName[FILENAMELEN];
  sprintf(notarizedFileName, "%s/abc.txt", TMP);
  char notRecFileName[FILENAMELEN];
  sprintf(notRecFileName, "%s/abc.snr", TMP);
  char valResultFileName[FILENAMELEN];
  sprintf(valResultFileName, "%s/result.txt", TMP);
  FILE *notarizedFile;
  FILE *notRecFile;
  FILE *valResultFile; // declare a FILE pointer 
  char c;
  char suretyNotarizeCommand[ATCOMLEN];
  sprintf(suretyNotarizeCommand, "./%s -n -s test.surety.com -u %s -p %s -d ",
          SURETY_BINARY,
//          "Arizona-Test1", "glyutnsVUU");
          ver_param->surety_user_name, ver_param->surety_password);
  char suretyValidateCommand[ATCOMLEN];
  sprintf(suretyValidateCommand, "./%s -n -s test.surety.com -u %s -p %s -d ",
          SURETY_BINARY,
//          "Arizona-Test1", "glyutnsVUU");
          ver_param->surety_user_name, ver_param->surety_password);
  strcat(suretyNotarizeCommand,notarizedFileName);
  strcat(suretyNotarizeCommand," -r ");
  strcat(suretyNotarizeCommand,notRecFileName);
  fprintf(stderr, "Command = %s\n", suretyNotarizeCommand);

  strcat(suretyValidateCommand,notarizedFileName);
  strcat(suretyValidateCommand," -r ");
  strcat(suretyValidateCommand,notRecFileName);
  strcat(suretyValidateCommand," > ");
  strcat(suretyValidateCommand,valResultFileName);

  char rmNotRecCommand[ATCOMLEN]="rm -f ";
  strcat(rmNotRecCommand,notRecFileName);
  //write hash_value to "abc.txt"
  notarizedFile = fopen(notarizedFileName,"w");
  int i;
  print_hash(ver_param->hash_value);

  if (fclose(notarizedFile)) {
    fprintf(stderr, "err source: dd\n");
  }
  //before each notarize, delete the rec file 
  if (system(rmNotRecCommand) == -1) {
      // error when run Notarizer from shell
    fprintf(stderr,"Can't run notarizer");
    exit(1);
  }
  printf("SURETY IS COMMING! Notarized Hash_value is:\n");
  print_hash(ver_param->hash_value);
  //now can set the next execution of Notarizer
  if (system(suretyNotarizeCommand) == -1) {
    // error when run Notarizer from shell
    //insErrEvent(dbc,"Can't run notarizer");  TODO: needs to include dbc
    fprintf(stderr,"Can't run notarizer");
    exit(1);
  }
}


void CommunicateWithDNS(VerificationParameter* ver_param) {
  switch (ver_param->method) {
    case LOCAL_VERIFICATION:
      CommunicateWithLocalDNS(ver_param);
      return;
    case SURETY_VERIFICATION:
      CommunicateWithSuretyDNS(ver_param);
      return;
    default:
      return;
  }
}


int VerifyByLocalDNS(VerificationParameter* ver_param) {
//  char hash_value[AUD_HASH_LEN];
//  struct timeval ttime;
  int n;
  NID nid, oldnid;
//  SQLHDBC mysqldbc;
//  char maxPreNotID[IDLEN];
  int verification_result = 1;

  CommunicateWithLocalDNS(ver_param);

  getNIDfromChain(ver_param->dbc, oldnid.nid, ver_param->notarization_id);
  if (memcmp(oldnid.nid, ver_param->nid_output.nid, AUD_HASH_LEN) != 0) {
    printf("Original Notarizer Nid:");
    print_hash(oldnid.nid);
    printf("New Nid:");
    print_hash(ver_param->nid_output.nid);
    verification_result = 1;
  } else {
    verification_result = 0;
  }
  return verification_result;
}


int VerifyBySurety(VerificationParameter* ver_param) {
  char valResultFileName[FILENAMELEN];
  sprintf(valResultFileName, "%s/result.txt", TMP);
  FILE *notarizedFile;
  FILE *notRecFile;
  FILE *valResultFile; // declare a FILE pointer 
  char c;
  int validation_result;
  CommunicateWithSuretyDNS(ver_param);
  //compare the result
  valResultFile = fopen(valResultFileName, "r");
  if(valResultFile==NULL) {
    printf("Error: can't open file.\n");
    // fclose(file); DON'T PASS A NULL POINTER TO fclose !! 
    return 1;
  } else {
    while(1) {     // keep looping...
      c = fgetc(valResultFile);
      if(c!=EOF) {
        if (c=='p'){
          validation_result = 0;
        }else{
          validation_result = 1;
        }
        // print the file one character at a time 
      }
      else {
        break;     // ...break when EOF is reached 
      }
    }
    fclose(valResultFile);
    return validation_result;
  }
}


int DoVerification(VerificationParameter* ver_param) {
  switch (ver_param->method) {
    case LOCAL_VERIFICATION:
      return VerifyByLocalDNS(ver_param);
    case SURETY_VERIFICATION:
      return VerifyBySurety(ver_param);
    default:
      return -1;
  }
}
