#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// ODBC use
#include <sql.h>
#include <sqlext.h>

#include "alg.h"
#include "common.h"

#include "dnsverifier.h"

//#define MAXSQLQUERY 256

void Usage() {
  printf("Usage:\n");
  printf("  notarizer db_path db_name binary_dir notarization_server\n");
  printf("OR:\n");
  printf("  notarizer db_path db_name binary_dir surety_username \
         surety_password\n");
  printf("Note: db_path must be an absolute path string.\n");
  printf("  binary_dir is the dir holding both validator and notarizer.\n");
  printf("  binary_dir must be an absolute path string.\n");
}


int main(int argc, char *argv[]) {
  SQLHENV env;
  SQLHDBC dbc;
  SQLRETURN ret; /* ODBC API return status */
  SQLCHAR outstr[1024];
  SQLSMALLINT outstrlen;
  SQLCHAR sqldsn[] = "DSN=mysql;";

/* Modified by Huilong */
   //check the arguments
  VerificationParameter ver_param;
  if (argc == 5) {
    ver_param.method = LOCAL_VERIFICATION;
    ver_param.binary_dir = argv[3];
    ver_param.server_name = argv[4];
    // note: right now the dbc field is not yet initialized, Notarize() should
    // do so.
  } else if (argc == 6) {
    ver_param.binary_dir = argv[3];
    ver_param.surety_user_name = argv[4];
    ver_param.surety_password = argv[5];
    ver_param.method = SURETY_VERIFICATION;
  } else {
    Usage();
    exit(1);
  }
   
/* end Huilong */
  printf("Begin Notarizer\n");
  InitConfigTable();
  strncpy(ConfigTable.Dbpath,argv[1],DBPATHLEN-1);
  strncpy(ConfigTable.Dbname,argv[2],DBPATHLEN-1);
  /* Allocate an environment handle */
  SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &env);
  /* We want ODBC 3 support */
  SQLSetEnvAttr(env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);
  /* Allocate a connection handle */
  SQLAllocHandle(SQL_HANDLE_DBC, env, &dbc);

  /* Connect to the DSN mydsn */
  /* You will need to change mydsn to one you have created and tested */
  //ret = SQLDriverConnect(dbc,(void *)1 , "DSN=mysql;", SQL_NTS,
  ret = SQLDriverConnect(dbc, (void *)1, sqldsn, SQL_NTS, outstr,
                         sizeof(outstr), &outstrlen, SQL_DRIVER_COMPLETE);
  if (SQL_SUCCEEDED(ret)) {
    printf("Connected\n");
    //    printf("Returned connection string was:\n\t%s\n", outstr);
  } else {
    fprintf(stderr, "Failed to connect in notarize\n");
    SQLINTEGER native;
    SQLCHAR state[7];
    SQLCHAR text[256];
    SQLSMALLINT txt_len;
    SQLSMALLINT type = SQL_HANDLE_DBC;
    SQLGetDiagRec(type, dbc, 1, state, &native, text, 256, &txt_len);
    fprintf(stderr, "%s %d: %s\n", state, native, text);
    /* free up allocated handles */
    SQLFreeHandle(SQL_HANDLE_DBC, dbc);
    SQLFreeHandle(SQL_HANDLE_ENV, env);
  }
  ReadConf(dbc);
  getStartDate(dbc);
  PrintConfigTable();
  ErrorCheck();

  Notarize(dbc, &ver_param);

  printf("Exit Notarizer\n");
  return 0;
}

