#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
/* Added by Huilong */
#include <netdb.h>
/* end Huilong */
#include <unistd.h>
#include <sys/times.h>

/* tBDB support */
#include "db.h"
typedef struct timeval TV;

/*BEECRYPT support*/
#include "sha1.h"

#include "red_black_tree.h"
#include "validator.h"
#include "common.h"
#include "dnsverifier.h"

#define MAXLINE 100
//#define ATCOMLEN 512

typedef struct sockaddr SA;

DB_ENV *dbenv;
DB *db;

typedef struct sockaddr_in SAI;

struct __ver_tol_entry {
    //TV time;
    struct timeval time;
    char hash_value[AUD_HASH_LEN];
    void *hash_param; /* hash function state */
    Heap_T tuple_heap; /* priority queue */
    int expected_seq; /* next expected sequence number */
    /*struct __ver_tol_entry *next;*/
};
typedef struct __ver_tol_entry VER_TOL_ENTRY;

struct __tuple {
    int seq; /* indexing sequence number in the priority tuque. */
    DBT data;
};

typedef struct __tuple TUPLE;

rb_red_blk_tree *tree; /* TOL */

#define time_cmp(at, bt) ((at.tv_sec==bt.tv_sec)? (at.tv_usec-bt.tv_usec): (at.tv_sec-bt.tv_sec))

void mydest(void *a) {free((VER_TOL_ENTRY *)a);}
int mycomp(const void *a, const void *b)
{return time_cmp((((VER_TOL_ENTRY *)a)->time), (((VER_TOL_ENTRY *)b)->time));}
//void myprint(const void *a) {printf("<%d,%d>", ((TV *)a)->tv_sec, ((TV *)a)->tv_usec);}
void myprint(const void *a) {printf("<%d,%d>", ((struct timeval *)a)->tv_sec, ((struct timeval *)a)->tv_usec);}
void myinfoprint(void *a) {printf("[%d]",((VER_TOL_ENTRY *)a)->expected_seq);}
//void myinfoprint(void *a) {return;}
void myinfodest(void *a) {return;}

void env_open(int join);
void db_open(char *name, int join);
void SendEmail(SQLHDBC dbc);

void WriteSynRegion(SQLHDBC dbc, char *FailureID,
                 char *TopLeftX, char *TopLeftY,
                 char *TopRightX, char *TopRightY,
                 char *BottomLeftX, char *BottomLeftY,
                 char *BottomRightX, char *BottomRightY
                 );


// modified by Man, 06/03/2007
//void MonoForensic(SQLHDBC dbc,  fail_flag, char *server, in_port_t port,char *maxPreNotID);
void MonoForensic(SQLHDBC dbc, VerificationParameter* ver_param,
                  in_port_t port,char *maxPreNotID);
//

void UpdateRegion(SQLHDBC dbc, char *failNotID,char *maxPreNotID,char *curTamperID);

void show_val_error()
{
    fprintf(stderr, "Error in validator.c!\n");
    exit(1);
}


void Usage() {
  printf("Usage:\n");
  printf("  validator db_path db_name notarization_server\n");
  printf("OR:\n");
  printf("  validator db_path db_name surety_username surety_password\n");
  printf("Note: db_path must be an absolute path string.\n");
}


int main(int argc, char *argv[])
{
    SQLHENV env;
    SQLHDBC mysqldbc;
    SQLRETURN mysqlret; /* ODBC API return status */
    SQLCHAR outstr[1024];
    SQLSMALLINT outstrlen;
    SQLCHAR sqldsn[] = "DSN=mysql;";

    NID nid, oldnid;
   // DBT key, data;
    DB_TXN *tid;
    DBC *taudbc;
    int tauret;
    VER_TOL_ENTRY *p;
    AUD_TXN_HASH_DATA txn_hash_data;
    time_t start_time;
    rb_red_blk_node *np;
//    int sockfd;
    int i;

/* Added by Huilong */
    struct hostent *serv_p;
//    char command[256]; //modified by Man 06/03/2003
    int  fail_flag = 0; // if flag is set, the validation fails
/* end Huilong */
    SAI servaddr;
    char hash_value[AUD_HASH_LEN];
    //TV ttime;
    struct timeval ttime;
    int n;

    in_port_t port;
    port = 11111;  //for test only

printf("Begin Validator\n");

    InitConfigTable();

/* Modified by Huilong */
   //check the arguments
//   if(argc!=4){
//      printf("Usage: %s db_path db_name notarization_server\n", argv[0]);
//      exit(1);
//   }

    VerificationParameter ver_param;
    if (argc == 4) {
      ver_param.method = LOCAL_VERIFICATION;
      ver_param.server_name = (char *) malloc (strlen(argv[3]));
      memcpy(ver_param.server_name, argv[3], strlen(argv[3]));
      // note: right now the dbc field is not yet initialized, Notarize() should
      // do so.
    } else if (argc == 5) {
      ver_param.surety_user_name = argv[3];
      ver_param.surety_password = argv[4];
      ver_param.method = SURETY_VERIFICATION;
    } else {
      Usage();
      exit(1);
    }

    strncpy(ConfigTable.Dbpath,argv[1],DBPATHLEN-1);
    strncpy(ConfigTable.Dbname,argv[2],DBPATHLEN-1);
/* end Huilong */

    /* Allocate an environment handle */
    SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &env);

    /* We want ODBC 3 support */
    SQLSetEnvAttr(env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);

    /* Allocate a connection handle */
    SQLAllocHandle(SQL_HANDLE_DBC, env, &mysqldbc);

    /* Connect to the DSN mydsn */
    /* You will need to change mydsn to one you have created and tested */
    //ret = SQLDriverConnect(dbc,(void *)1 , "DSN=mysql;", SQL_NTS,
    mysqlret = SQLDriverConnect(mysqldbc,(void *)1 , sqldsn, SQL_NTS,
                 outstr, sizeof(outstr), &outstrlen, SQL_DRIVER_COMPLETE);
    if (SQL_SUCCEEDED(mysqlret)) {
        printf("Connected\n");
    //    printf("Returned connection string was:\n\t%s\n", outstr);
    } else {
        fprintf(stderr, "Failed to connect\n");

        /* free up allocated handles */
        SQLFreeHandle(SQL_HANDLE_DBC, mysqldbc);
        SQLFreeHandle(SQL_HANDLE_ENV, env);
    }


    /* Read information from DB and put them into the global variable "ConfigTable" */
    ReadConf(mysqldbc);
/* Added by Huilong */
    if(checkExistingTampering(mysqldbc)==1){
        printf("Tampering existing! Validator quit!\n");
        exit(1);
    }
/* End Huilong */
    getStartDate(mysqldbc);
    PrintConfigTable();
    ErrorCheck();

    // get Startdate/time of Validation Interval
    //char prevdate[30],prevtime[30];
    char prevactualdate[DATELEN],prevactualtime[DATELEN], prevactualms[MSLEN];
    char stopactualdate[DATELEN],stopactualtime[DATELEN];
    char stopSchDate[TIMESTPLEN],stopActDate[TIMESTPLEN],stopactualms[MSLEN];

    char maxPreNotID[IDLEN];

    getStart(prevactualdate,prevactualtime,prevactualms);

    getStop(mysqldbc,stopSchDate,stopActDate,stopactualms,maxPreNotID);

    // convert to date,time format
    getDate(stopActDate,stopactualdate,stopactualtime);

printf("Start date:%s %s and %s\n",prevactualdate,prevactualtime,prevactualms);
printf("Stop date:%s %s and %s\n",stopactualdate,stopactualtime,stopactualms);

    struct timeval start_val,stop_val;

//    start_val = mkCalTime(prevactualdate,prevactualtime,prevactualms);
    start_val.tv_sec = BEGINNING;  /*#define BEGINNING (LONG_MAX - 1) a very big number*/
    start_val.tv_usec = 0;

    stop_val = mkCalTime(stopactualdate,stopactualtime,stopactualms);


//external sorting start: need to be replaced by external sorting code to sort the db.  Starting from here. 

    // rebuild TOL for tauBerkeley DB according to the db source
    tree = RBTreeCreate(mycomp, mydest, myinfodest, myprint, myinfoprint);
    rebuild_tol(start_val,stop_val);
print_tol();
printf("Begin recalculate hashing.\n");

    np = RBMin(tree);
    p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
    memset(hash_value, 0, AUD_HASH_LEN);

    /* Rechain the txn's in TOL till the tail time "ttime"
     * and put the tail hash_value in "hash_value" for verification.
     */
    /* If tree is null,no hash value recalculated */
    if (p == NULL) {
 	printf("No tuples in DB Yet.\n");
        goto finished;
    }

gettimeofday(&ttime, NULL);

    while (p != NULL) {
        if (time_cmp(p->time, ttime) == 0)
            break;
        printf("TOL:txn_time(%d,%d)\n",
                   p->time.tv_sec, p->time.tv_usec);

//   	printf("    start hash_value = ");
//        print_hash(hash_value);

//        printf("    txn_hash =");
//        print_hash(p->hash_value);

        memcpy(txn_hash_data.prev_hash_value, hash_value, AUD_HASH_LEN);
        txn_hash_data.time = p->time;
        memcpy(txn_hash_data.hash_value, p->hash_value, AUD_HASH_LEN);
        if (__aud_hash(&txn_hash_data, sizeof(AUD_TXN_HASH_DATA), hash_value) != 0)
            goto err;

//rui        printf("!!! final:   tail_hash=");
//rui        print_hash(hash_value);

//        if (time_cmp(p->time, ttime) == 0)
//            break;

        np = TreeSuccessor(tree, np);

        if (tree->nil == np ) {
            break;
        }
        p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
    }
    printf("!!! final:   tail_hash=");
    print_hash(hash_value);

//external sorting ends: need to be replaced by external sorting code to sort the db.  Ends here. 

    ver_param.dbc = mysqldbc;
    ver_param.notarization_id = maxPreNotID;
    ver_param.hash_value = (char*) malloc (AUD_HASH_LEN);
    memcpy(ver_param.hash_value, hash_value, AUD_HASH_LEN);
    memcpy(&(ver_param.ttime), &ttime, sizeof(ttime));
    int validation_result = DoVerification(&ver_param);
    free(ver_param.hash_value);

/*
    // Establish UDP 
    char notarizedFileName[FILENAMELEN];
    sprintf(notarizedFileName, "%s/abc.txt", PATH);
    char notRecFileName[FILENAMELEN];
    sprintf(notRecFileName, "%s/abc.snr", PATH);
    char valResultFileName[FILENAMELEN];
    sprintf(valResultFileName, "%s/result.txt", PATH);
    FILE *notarizedFile;
    FILE *notRecFile;
    FILE *valResultFile; // declare a FILE pointer  

    char c;

    char suretyNotarizeCommand[ATCOMLEN];
    sprintf(suretyNotarizeCommand,
            "%s/testnc -n -s test.surety.com -u \
            Arizona-Test1 -p glyutnsVUU -d ",
            PATH);

    char suretyValidateCommand[ATCOMLEN];
    sprintf(suretyValidateCommand,
            "%s/testnc -v -s test.surety.com -u \
            Arizona-Test1 -p glyutnsVUU -d ",
            PATH);

    strcat(suretyNotarizeCommand,notarizedFileName);
    strcat(suretyNotarizeCommand," -r ");
    strcat(suretyNotarizeCommand,notRecFileName);

    strcat(suretyValidateCommand,notarizedFileName);
    strcat(suretyValidateCommand," -r ");
    strcat(suretyValidateCommand,notRecFileName);
    strcat(suretyValidateCommand," > ");
    strcat(suretyValidateCommand,valResultFileName);

    char rmNotRecCommand[ATCOMLEN]="rm -f ";
    strcat(rmNotRecCommand,notRecFileName);

    char validation_result = 0;


    //write hash_value to "abc.txt"
    notarizedFile=fopen(notarizedFileName,"w");

    for (i=0; i<AUD_HASH_LEN; i++)
    {
    	fprintf(notarizedFile,"%02X",0xff & hash_value[i]); 
    } 

    if (fclose(notarizedFile)) {
 	show_val_error();
    }

    fprintf(stdout, "SURETY IS COMMING! Notarized Hash_value is:\n");
    print_hash(hash_value);

    //validate "abc.txt"
    if (system(suretyValidateCommand) == -1) {
        // error when run Notarizer from shell
        //insErrEvent(dbc,"Can't run notarizer");  TODO: needs to include dbc
        fprintf(stderr,"Can't run validator");
        exit(1);
    }

    //compare the result
    valResultFile = fopen(valResultFileName, "r"); 

  if(valResultFile==NULL) {
    printf("Error: can't open file.\n");
    // fclose(file); DON'T PASS A NULL POINTER TO fclose !! 
    return 1;
  }
  else {
    
    while(1) {     // keep looping... 
      c = fgetc(valResultFile);
      if(c!=EOF) {
	if (c=='p'){
          validation_result = 0;  
	}else{
          validation_result = 1;  	
	}
        // print the file one character at a time 
      }
      else {
        break;     // ...break when EOF is reached
      }
    }

    fclose(valResultFile);

  }
*/



#if 0
    memset(&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons(port);

/* Modified by Huilong */
//    inet_pton(AF_INET, argv[2], &servaddr.sin_addr);
    serv_p = gethostbyname(argv[3]);
    bcopy(serv_p->h_addr, &servaddr.sin_addr, serv_p->h_length);
/* end Huilong */

    sockfd = socket(AF_INET, SOCK_DGRAM, 0);

    sendto(sockfd, hash_value, AUD_HASH_LEN, 0,
               (SA *)&servaddr, sizeof(servaddr));

    printf("Request sent: ttime=(%d, %d)\n",
               ttime.tv_sec, ttime.tv_usec);
    printf("              thash=");
    print_hash(hash_value);

    n = recvfrom(sockfd, &rnid, sizeof(rnid), 0, NULL, NULL);
    printf("    N-id received: n=%d, ntime=(%d, %d)\n",
               rnid.ntime.tv_sec, rnid.ntime.tv_usec);

    printf("!!!!Nid recalculated:");
    print_hash(rnid.nid);

        /* Compare the N-id from the N-serv with the most recent Notarizer NID */

	getNIDfromChain(mysqldbc,oldnid.nid,maxPreNotID);
     
printf("Original Notarizer Nid:");
    print_hash(oldnid.nid);

#endif
        //if (memcmp(oldnid.nid,rnid.nid,AUD_HASH_LEN) != 0) {
        if (validation_result != 0) {
// experiment walk around!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//      if (memcmp(rnid.nid, nid.nid, AUD_HASH_LEN) != 0) {
            printf("  <<< Discrepancy found here!!! >>>\n");
     /*       printf("    Data in the nHistory:\n");
            printf("        ttime=(%d, %d)\n", ttime.tv_sec, ttime.tv_usec);
            printf("        ntime=(%d, %d)\n", rnid.ntime.tv_sec, rnid.ntime.tv_usec);
            printf("        hash=");
            print_hash(rnid.nid);
            printf("    Data recomputed from the database:\n");
            printf("        hash=");
            print_hash(hash_value);
     */
/* Added by Huilong */
            fail_flag = 1;
         // send email to the DBA
            SendEmail(mysqldbc);
/* end Huilong */

/* Modified by Huilong */
            //exit (1);
/* End Huilong */
        } else {
            printf("=====>Verification check at txn_time (%d, %d) is OK!!!<=====\n\n\n",
                   ver_param.ttime.tv_sec, ver_param.ttime.tv_usec);
        }

    //    memcpy(hash_value, rnid.nid, AUD_HASH_LEN);
        //free(data.data);
//    }

  finished:
    {
        struct tms TimeReading;
        times(&TimeReading);
        printf("CPU Time=%f sec\n", ((TimeReading.tms_stime + TimeReading.tms_utime) / 100.0));
    }
    printf("IO=%d\n", __aud_io(1));

//    printf("Verification successfully done. No discrepancy found!!! :) :) :)\n");

    printf("Total time=%d\n", time(NULL) - start_time);

    // insert one V event into Event table
   SQLCHAR insEvent[MAXINSLEN] = "insert into Event (ScheduledDate,ActualDate,ActualMilliseconds,Path,Name,Kind) VALUES ('";

    strcat(insEvent,stopSchDate);
    strcat(insEvent,"','");

    // find current time
    struct timeval current_t;
    char curValActDate[TIMESTPLEN];
    gettimeofday(&current_t,NULL);
    strftime(curValActDate,30,"%Y-%m-%d %X",localtime(&(current_t.tv_sec)));

    strcat(insEvent,curValActDate);
    strcat(insEvent,"',");

    char curValActMs[MSLEN];
    sprintf(curValActMs,"%d",current_t.tv_usec);
    strcat(insEvent,curValActMs);
    strcat(insEvent,",'");

    strcat(insEvent,ConfigTable.Dbpath);
    strcat(insEvent,"','");
    strcat(insEvent,ConfigTable.Dbname);
    strcat(insEvent,"','V')");

printf("\nval,insEvent:[%s]\n",insEvent);
    // execute to insert Event SQL
    exeSqlQuery(mysqldbc,insEvent);

    printf("  Validation-Event stored in the Event table.\n");

    // insert 'T' event if tampering happens
    if(fail_flag){
       // insert one tuple for this failure in the Event table

        SQLCHAR insFailure[MAXINSLEN] = "insert into Event (ActualDate,ActualMilliseconds,Path,Name,Kind) VALUES ('";
        strcat(insFailure,curValActDate);
        strcat(insFailure,"','");
        strcat(insFailure,curValActMs);
        strcat(insFailure,"','");
        strcat(insFailure,ConfigTable.Dbpath);
        strcat(insFailure,"','");
        strcat(insFailure,ConfigTable.Dbname);
        strcat(insFailure,"','T')");

printf("Insert an Tampering Event:[%s]\n",insFailure);

        // execute to insert Failure Event SQL
        exeSqlQuery(mysqldbc,insFailure);

printf("\n Tampering Event stored in the Event table.\n");

	if ( ConfigTable.Algorithm != NULL && 
		strcmp(ConfigTable.Algorithm,"Monochromatic") == 0) {
	    // modified by Man, 06/03/2007
	    // MonoForensic(mysqldbc,fail_flag,argv[3],port,maxPreNotID);
            MonoForensic(mysqldbc, &ver_param, port, maxPreNotID);
	}
	else if (ConfigTable.Algorithm != NULL && 
		strcmp(ConfigTable.Algorithm,"A3D") == 0 ) {
	    A3DForensic(mysqldbc,argv[3],port,maxPreNotID);
	}
    }
    else {
	// may need add more partial chains depending on the algorithm
    	if (ConfigTable.Algorithm != NULL  && 
			strcmp(ConfigTable.Algorithm,"A3D") == 0 ) {
	    // add partial chains in A3D
	    A3DPartialChain(mysqldbc,argv[3],port);
	}

    }

    // destroy TOL tree
    TreeDestHelper(tree, tree->root);
    //for debug only
//    __stp_timestop();
//    __stp_timereport();
printf("Exit valiator\n");
    return 0;
  err:
    fprintf(stderr, "Error in main()!\n");
    exit(1);
//}


    //Sort_tuples();

    return 0;
}



// convert date/time/ms into "struct timeval"
struct timeval mkCalTime(char *date, char *time, char *ms)
{
    struct tm oldtime;
    struct timeval oldtv;
    time_t oldtt;
    int i=0,j=0;
    //char hourchar[TIMELEN],minchar[TIMELEN],secchar[TIMELEN],yearchar[TIMELEN],monthchar[TIMELEN],daychar[TIMELEN];

    char totaltime[100];

    bzero(totaltime,100);
    strcat(totaltime,date);
    strcat(totaltime,time);

    strptime(totaltime,"%Y-%m-%d %T", &oldtime);

    oldtt = mktime(&oldtime);

    oldtv.tv_sec = oldtt;
    oldtv.tv_usec = atoi(ms);

//printf("*****oldtime in string:[%s],[%s],[%s]\n",date,time,ms);
//printf("********oldtime.tv_sec[%d],oldtime.tv_usec[%d]\n",oldtv.tv_sec,oldtv.tv_usec);

    return oldtv;

}

// get this validation event start time
void getStart(char *prevactualdate, char *prevactualtime, char *prevactualms)
{
    bzero(prevactualdate,DATELEN);
    bzero(prevactualtime,DATELEN);
    bzero(prevactualms,MSLEN);

    strcpy(prevactualdate,ConfigTable.StartDate);
    strcpy(prevactualtime,ConfigTable.StartTime);
    strcpy(prevactualms,"0");
}

/*void getStart(SQLHDBC dbc,char *prevdate, char *prevtime, char *prevactualdate, char *prevactualtime, char *prevactualms)
{
    char preValSchDate[TIMESTPLEN],preValActDate[TIMESTPLEN];

    // if this is the first run of Validator,set this value to 0

    if (!getMaxValEvent(dbc, preValSchDate,preValActDate,prevactualms)) {
        strcpy(prevdate,ConfigTable.StartDate);
        strcpy(prevtime,ConfigTable.StartTime);
        strcpy(prevactualdate,prevdate);
        strcpy(prevactualtime,prevtime);
        strcpy(prevactualms,"0");
    } else {
        getDate(preValSchDate,prevdate,prevtime);
        getDate(preValActDate,prevactualdate,prevactualtime);
    }

}
*/


// get this validation event stop time
void getStop(SQLHDBC dbc, char *stopSchDate,char *stopActDate, char *stopactualms, char *stopEventID)
{
    char maxNotID[IDLEN]; 

    getMaxNotID(dbc, maxNotID);

    getPreNotEvent(dbc, stopEventID,stopSchDate, stopActDate, stopactualms, maxNotID);

}


/*
void getStop(SQLHDBC dbc, char *stopactualdate, char *stopactualtime, char *stopactualms, int *EventID)
{
    char stopActDate[TIMESTPLEN],stopSchDate[TIMESTPLEN]; 

    if (!getMaxNotEvent(dbc,stopSchDate,stopActDate,stopactualms)) {
 	insErrEvent(dbc,"Validator runs before Notarizer");
	fprintf(stderr, "Validator runs before Notarizer!\n");
	exit (1);
    }

    getDate(stopActDate,stopactualdate,stopactualtime);

    char maxNotID[IDLEN];
    getMaxNotID(dbc,maxNotID);
    *EventID = atoi(maxNotID);
}
*/

void getHashvalueFromChain(SQLHDBC dbc,char *hash_value, char *notEventID)
{
    SQLHSTMT stmt;
    SQLRETURN ret; /* ODBC API return status */
    SQLSMALLINT columns; /* number of columns in result-set */
    int row = 0;
    SQLINTEGER indicator;
    char buf[512];
    SQLUSMALLINT i=1;
    SQLCHAR getHash[100] = "select Hashcode from Chain where Color='black' AND EventID=";
    int FindEvent = 0;

    //int oldstrlen = strlen(getNID);
    //strncat(getNID,hash_value,AUD_HASH_LEN);
    //getNID[oldstrlen+AUD_HASH_LEN+1] = '\0';
    strcat(getHash,notEventID);

printf("\ngetHashvalueFromChain:[%s]\n",getHash);
    /* Allocate a statement handle */
    SQLAllocHandle(SQL_HANDLE_STMT, dbc, &stmt);

    //get Configuration Column names
    ret = SQLExecDirect(stmt,getHash,SQL_NTS);
    printf("getHashvaluefromChain:%s\n",getHash);

    bzero(hash_value,AUD_HASH_LEN);
    if (ret == SQL_SUCCESS) {
        while (1) {
            ret = SQLFetch(stmt);
            if (ret == SQL_ERROR || ret == SQL_SUCCESS_WITH_INFO) {
                show_val_error();
            }

            if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO) {
                FindEvent = 1;

                SQLUSMALLINT i = 1;

                /* How many columns are there */
                SQLNumResultCols(stmt, &columns);
               /* Loop through the columns */
                for (i = 1; i <= columns; i++) {

                    /* retrieve column data as a string */
                    if (SQL_SUCCEEDED(SQLGetData(stmt, i, SQL_C_CHAR,
                              buf, sizeof(buf), &indicator))) {
                            str_to_value(buf,2*AUD_HASH_LEN,hash_value,AUD_HASH_LEN);
                    }
                }
            } else {
                break;
            }
        }
   }

   if (!FindEvent) {
        insErrEvent(dbc,"Can't find hashvalue for this Event\n");
        fprintf(stderr,"Can't find hashvalue for this Event\n");
	exit(1);
   }
}


/*
void getNIDfromChain(SQLHDBC dbc,char *nid_value, char *EventID)
{
    SQLHSTMT stmt;
    SQLRETURN ret; // ODBC API return status 
    SQLSMALLINT columns; // number of columns in result-set 
    int row = 0;
    SQLINTEGER indicator;
    char buf[512];
    SQLUSMALLINT i=1;
    SQLCHAR getNID[SQLQUERY] = "select NotarID from Chain where Color='black' AND EventID=";
    int FindEvent = 0;

    //int oldstrlen = strlen(getNID);
    //strncat(getNID,hash_value,AUD_HASH_LEN);
    //getNID[oldstrlen+AUD_HASH_LEN+1] = '\0';

    strcat(getNID,EventID);

    // Allocate a statement handle 
    SQLAllocHandle(SQL_HANDLE_STMT, dbc, &stmt);

    //get Configuration Column names
    ret = SQLExecDirect(stmt,getNID,SQL_NTS);
    printf("getNIDfromChain:%s\n",getNID);

    bzero(nid_value,AUD_HASH_LEN);
    if (ret == SQL_SUCCESS) {
        while (1) {
            ret = SQLFetch(stmt);
            if (ret == SQL_ERROR || ret == SQL_SUCCESS_WITH_INFO) {
                show_val_error();
            }

            if (ret == SQL_SUCCESS || ret == SQL_SUCCESS_WITH_INFO) {
		FindEvent = 1;

                SQLUSMALLINT i = 1;

                // How many columns are there 
                SQLNumResultCols(stmt, &columns);

                // Loop through the columns 
                for (i = 1; i <= columns; i++) {

                    // retrieve column data as a string
                    if (SQL_SUCCEEDED(SQLGetData(stmt, i, SQL_C_CHAR,
                              buf, sizeof(buf), &indicator))) {
			    str_to_value(buf,2*AUD_HASH_LEN,nid_value,AUD_HASH_LEN);
//sprintf(nid_value,"%s",nid_value);
//			strcpy(nid_value,buf);
                    }
                }
            } else {
                break;
            }
        }
   }

   if (!FindEvent) {
        insErrEvent(dbc,"Can't find NID for this Event\n");
	fprintf(stderr,"Can't find NID for this Event\n");
	exit(1);
   }

}
*/


static
int print_tol()
{
    int i = 0;
    VER_TOL_ENTRY *p;
    rb_red_blk_node *np;
    printf("+++++++++++++++++++++ Rebuit TOL +++++++++++++++++\n");
    np = RBMin(tree);
    p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
    while (p != NULL) {
        /*p = (VER_TOL_ENTRY *)(np->info);
        if (p == NULL)
            break;
*/
//rui        printf("Time=(%d, %d), hash=", p->time.tv_sec, p->time.tv_usec);
        print_hash(p->hash_value);
        np = TreeSuccessor(tree, np);
	if (tree->nil == np ) {
	    break;
        }
	p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);

    }
    printf("++++++++++++++++++++++++++++++++++++++++++++++++++\n");
    return 0;
}

static
int cmp(const void *a, const void *b) {
    TUPLE *t1 = (TUPLE *)a,
        *t2 = (TUPLE *)b;
    return (t1->seq - t2->seq);
}

int timebound = 0;


int rebuild_tol(struct timeval start_val,struct timeval stop_val)
{
    //DB *db;
    int ret;
    u_int32_t flags;
    DBT key, data;
    TUPLE *tuple1, *tuple2;
//    struct __ttdbc *dbcp = NULL;
    DBC *dbcp;
    DB_TXN *tid;
    int start1, stop1;
    //TV start, stop, ret_start, ret_stop;
    struct timeval start, stop, ret_start, ret_stop;
    VER_TOL_ENTRY *p, *p1, *tmp;
    //char digest[AUD_HASH_LEN];
    TTDATA *ttdata;
    int counter = 0;
    rb_red_blk_node *np;


    flags = DB_TEMPORAL;


    printf("ENTER rebuild_tol()\n");

    // open tauBerkeley DB to be verified

    env_open(0);
    printf("Env opened\n");

    db_open(ConfigTable.Dbname, 1);
    printf("%s DB opened\n",ConfigTable.Dbname);


    /* Begin the transaction. */
    if ((ret = txn_begin(dbenv, NULL, &tid, 0)) != 0) {
         dbenv->err(dbenv, ret, "DB_ENV->txn_begin");
         exit (1);
    }

    if (db->cursor(db, tid, &dbcp, 0) != 0) {
        dbenv->err(dbenv, ret, "DB->cursor");
        exit (1);
    }

    /* Initialize the key/data return pair. */
//printf("sizeof (key):%d,sizeof(data):%d,sizeof(DBT):%d\n",sizeof(key),sizeof(data),sizeof(DBT));
    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
//    data.flags = DB_DBT_MALLOC;

    /* Scan the database for given Validation Event Interval */
    start.tv_sec = start_val.tv_sec;
    start.tv_usec = start_val.tv_usec;
    stop.tv_sec = stop_val.tv_sec;
    stop.tv_usec = stop_val.tv_usec;

printf("Validation start time (%d,%d), stop time (%d, %d)\n", start_val.tv_sec, start_val.tv_usec, stop_val.tv_sec, stop_val.tv_usec);

    while ((ret = (dbcp->c_get_nonsequenced(dbcp, &key, &data,
                   start, stop, DB_NEXT, &ret_start, &ret_stop))) == 0)
    {
//        printf("    rebuild_tol(): Found a tuple: %s\n", (char *)data.data);
//printf("<%d,%d>", ret_start.tv_sec, ret_start.tv_usec);

//printf("--------print RDBLKTREE!--------\n");
//RBTreePrint(tree);
//printf("-----end print------\n");


        counter++;
//        continue;


        /* Make two copies of the "data" DBT. One is put in the
         *   starting transaction's tuple priority queue. The other
         *   is put in the stopping transaction's tuple priority queue.
         * When any copy of the "data" tuple is hashed, free its space.
         */
        tuple1 = (TUPLE *)malloc(sizeof(TUPLE));
        tuple1->data.data = (void *)malloc(data.size);
        tuple1->data.size = data.size;
        tuple1->seq = ((TTDATA *)(data.data - sizeof(TTDATA)))->start_seq;
        memcpy(tuple1->data.data, data.data, data.size);

        tuple2 = (TUPLE *)malloc(sizeof(TUPLE));
        tuple2->data.data = (void *)malloc(data.size);
        tuple2->data.size = data.size;
        tuple2->seq = ((TTDATA *)(data.data - sizeof(TTDATA)))->stop_seq;
        memcpy(tuple2->data.data, data.data, data.size);

//        free(data.data);

        /* Assertion: if the stop seq number is 0, the stop time must be UC! */
        if ((tuple2->seq == 0) && !is_until_changed(ret_stop)) {
            fprintf(stderr,
                    "Fail: Assertion: if the stop seq number is 0, the stop time must be UC!\n");
            goto err;
        }


        if (ret_start.tv_sec < timebound)
            goto ignorestart;

/*
printf("--------print RDBLKTREE!--------\n");
RBTreePrint(tree);
printf("-----end print------\n");
printf("call RBExactQuery of key<%d,%d>", ret_start.tv_sec, ret_start.tv_usec);
*/

        np = RBExactQuery(tree, &ret_start);
        if (np == NULL)
	{
            p1 = NULL;
//	    printf("this is the first node of this key in RB tree.\n");
	}
        else
	{
            p1 = (VER_TOL_ENTRY *)(np->info);
	    printf("this node exists in RB tree.\n");
	}

        if (p1 == NULL) {
            /* Create a new node on tol */
            if ((tmp = malloc(sizeof(VER_TOL_ENTRY))) == NULL)
                goto err;
            tmp->time = ret_start;
            memset(tmp->hash_value, 0, AUD_HASH_LEN);

            sha1Param *param;
            param = (sha1Param *)malloc(sizeof(sha1Param));
            if (sha1Reset(param))
                goto err;
            tmp->hash_param = param;

            // The max number of tuples per transaction
            #define MAX_TUPLES 200
            tmp->tuple_heap = Heap_new(MAX_TUPLES, cmp, NULL);
            tmp->expected_seq = 1;	/*next expected seq num */

            if ((np = RBTreeInsert(tree, &(tmp->time), tmp)) == NULL) {
                printf("RBTreeInsert error\n");
                exit(1);
            }
//              goto ignorestart;
            p1 = (VER_TOL_ENTRY *)(np->info);
        }

        /* Here p1 points to the node which has the same time
         * as ret_start.
         */

        /* Put the tuple in the "tuple priority queue" of the
         *   transaction.
         */
        if (Heap_insert(p1->tuple_heap, (void *)tuple1) != 0)
            goto err;

//printf("start time: tuple seq=%d put in queue, p1->expected_seq=%d\n",
//       tuple1->seq, p1->expected_seq);

        /* See if we can incrementally hash some tuples for the transaction until
         *   the next hole of unseen tuple.
         */

	/* A node stores all the tuples of the same transaction, because they have the same txn time*/
        while (1)
        {
            TUPLE *tuple_tmp;
            tuple_tmp = (TUPLE *)Heap_top(p1->tuple_heap);

            /* When tuple_tmp is NULL, there is no tuple in the queue */
            if (tuple_tmp == NULL)
                break;

            if (tuple_tmp->seq != p1->expected_seq)
                break;

//printf("(hash start: txn_time=(%d,%d) seq=%d, expected=%d)\n",
//       p1->time.tv_sec, p1->time.tv_usec,
//       tuple_tmp->seq, p1->expected_seq);
            /* This tuple's start_seq is our expected next sequence number.
             *   So we incrementally hash it to this transaction.
             *   And remove it from the transaction's priority queue;
             */
            if (sha1Update(p1->hash_param, tuple_tmp->data.data, tuple_tmp->data.size))
                    goto err;
            Heap_extract_min(p1->tuple_heap);

            /* After hashing this tuple, we can free the space of this tuple */
            free(tuple_tmp->data.data);
            free(tuple_tmp);

            p1->expected_seq ++;
        }

        ignorestart:

        /* Search for stop time on the rebuilt tol */
        if (!is_until_changed(ret_stop))
        {
            if (ret_stop.tv_sec < timebound)
		return -1;
                //goto ignorestop;
//printf("#"); fflush(stdout);
            np = RBExactQuery(tree, &ret_stop);
            if (np == NULL)
                p1 = NULL;
            else
                p1 = (VER_TOL_ENTRY *)(np->info);

            if (p1 == NULL) {
                /* Create a new node on tol */
                if ((tmp = malloc(sizeof(VER_TOL_ENTRY))) == NULL)
                    goto err;
                tmp->time = ret_stop;
                memset(tmp->hash_value, 0, AUD_HASH_LEN);
                {
                    sha1Param *param;
                    param = (sha1Param *)malloc(sizeof(sha1Param));
                    if (sha1Reset(param))
                        goto err;
                    tmp->hash_param = param;
                }
                tmp->tuple_heap = Heap_new(MAX_TUPLES, cmp, NULL);
                tmp->expected_seq = 1;

                if ((np = RBTreeInsert(tree, &(tmp->time), tmp)) == NULL) {
                    printf("RBTreeInsert error\n");
                    exit(1);
                }
                p1 = (VER_TOL_ENTRY *)(np->info);
            }

             /* Here p1 points to the node which has the same time
              * as ret_start
              */

             /* Put the tuple in the "tuple priority queue" of the
              *   transaction.
              */
            if (Heap_insert(p1->tuple_heap, (void *)tuple2) != 0)
                goto err;

//printf("stop time: tuple seq=%d put in queue, p1->expected_seq=%d\n",
//       tuple2->seq, p1->expected_seq);
/*
printf("stop time: tuple seq=%d put in queue, p1->expected_seq=%d\n",
       tuple1->seq, p1->expected_seq);
*/
            /* See if we can incrementally hash some tuples until
             *   the next hole of unseen tuple.
             */
            while (1)
            {
                TUPLE *tuple_tmp;
                tuple_tmp = (TUPLE *)Heap_top(p1->tuple_heap);

                /* When tuple_tmp is NULL, there is no tuple in the queue */
                if (tuple_tmp == NULL)
                    break;

                if (tuple_tmp->seq != p1->expected_seq)
                    break;

//printf("(hash stop: txn_time=(%d,%d) seq=%d, expected=%d)\n",
//       p1->time.tv_sec, p1->time.tv_usec,
//       tuple_tmp->seq, p1->expected_seq);

                /* This tuple's start_seq is our expected next sequence number.
                 *   So we incrementally hash it to this transaction.
                 *   And remove it from the transaction's priority queue;
                 */
                if (sha1Update(p1->hash_param, tuple_tmp->data.data, tuple_tmp->data.size))
                    goto err;
                Heap_extract_min(p1->tuple_heap);

                /* After hashing this tuple, we can free the space of this tuple */
                free(tuple_tmp->data.data);
                free(tuple_tmp);

                p1->expected_seq ++;
            }
        //    ignorestop:
        }

/* added by Huilong */
     memset(&key, 0, sizeof(key));
     memset(&data, 0, sizeof(data));
//     data.flags = DB_DBT_MALLOC;
/* End Huilong */
    }
    printf("ret = %d\n", ret);
    dbenv->err(dbenv, ret, "population get_nonsequenced");

    if (dbcp->c_close(dbcp) != 0) {
        dbenv->err(dbenv, ret, "DBC->c_close");
        exit (1);
    }

    if ((ret = txn_commit(tid, 0)) != 0) {
        dbenv->err(dbenv, ret, "DB_TXN->commit");
        exit (1);
    }

/*    if (db->close(db, 0) != 0) {
        dbenv->err(dbenv, ret, "db->close: %s", ConfigTable.Dbname);
        exit (1);
    }

    /* Tuple sorting and incremental hashing is done.
     *   Now we finish the digest and copy the digest into the
     *   p->hash_value of each entry of the result TOL.
     *   Also free the space used to store the hash states (param).
     *   The tuple heap (priority queue) is deallocated too!*/

/*
    p = tol;
    while ((p = p->next) != NULL) {
            sha1Param *param;

            param = p->hash_param;
            if (param == NULL)
                goto err;

            if (sha1Digest(param, p->hash_value))
                goto err;

            free(param);
            p->hash_param = NULL;

            Heap_free(&(p->tuple_heap));
    }
*/

/********************************/
/*
printf("====print RDBLKTREE!--------\n");
RBTreePrint(tree);
printf("-----end print------\n");
print_tol();
*/

int i = 0;
    np = RBMin(tree);
    p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
    while (p != NULL) {
        sha1Param *param;

//printf(".");fflush(stdout);

/*
        p = (VER_TOL_ENTRY *)(np->info);
        if (p == NULL)
            break;
*/

        param = p->hash_param;
        if (param == NULL)
        goto err;

/*
printf("before sha1digest [%d]\n",i++);
print_hash(p->hash_value);
*/


        if (sha1Digest(param, p->hash_value))
	        goto err;

/*
printf("After sha1digest\n");
print_hash(p->hash_value);
*/


        free(param);
        p->hash_param = NULL;

        Heap_free(&(p->tuple_heap));

        np = TreeSuccessor(tree, np);

        if (tree->nil == np ) {
            break;
        }

        p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
    }

/***********************************/
    printf("******************>>>>> COUNTER = %d\n", counter);
//printf("print RDBLKTREE!>>>>\n");
//RBTreePrint(tree);

/* Added by Huilong */
//    dbenv->close(dbenv, 0);
/* end Huilong */

    return 0;
  err:
    fprintf(stderr, "ERROR in rebuild_tol\n");
    exit (1);
}


void env_open(int join)
{
    int ret;
    u_int32_t flags, flags1, flags0;
    int num_retry = 0;
    char pathname[256];

    flags0 = DB_CREATE | DB_PRIVATE | DB_INIT_LOCK | DB_INIT_LOG |
        DB_INIT_MPOOL | DB_INIT_TXN /*| DB_RECOVER*/ | DB_THREAD;
    flags1 = DB_JOINENV | DB_THREAD;

    if (join == 1)
        flags = flags1;
    else
        flags = flags0;

    /* Create the environment handle. */
    if ((ret = db_env_create(&dbenv, 0)) != 0) {
        fprintf(stderr,
                "txnapp: db_env_create: %s\n", db_strerror(ret));
        exit (1);
    }

    /* Set up error handling. */
    dbenv->set_errpfx(dbenv, "txnapp");

    /*
     * Open a transactional environment:
     *  create if it doesn't exist
     *  free-threaded handle
     *  run recovery
     *  read/write owner only
     */
  retry:
//for debug only
/*if (flags == flags1)
        printf("flags1\n");
else if (flags == flags0)
        printf("flags0\n"); */
    /* Huilong hard coded path name 5/3/2007*/
    pathname[0]='\0';
    sprintf(pathname, "%s", ConfigTable.Dbpath);
    //sprintf(pathname, "/cs/timecenter/tau/audit/auditutilities_mz_6_6_2007/%s", ConfigTable.Dbpath);
    if ((ret = dbenv->open(dbenv, /*ConfigTable.Dbpath*/pathname, flags,

                           0664)) != 0) {
        sleep(1);
       if (num_retry++ < 10)
            goto retry;
        dbenv->err(dbenv, ret, "dbenv->open: %s", ConfigTable.Dbpath);
        exit (1);
    }
}

int key_comp(DB *dbp, const DBT *a, const DBT *b)
{
    struct timeval *at = a->data,
    *bt = b->data;
    if (at->tv_sec < bt->tv_sec)
        return -1;
    else if (at->tv_sec > bt->tv_sec)
        return 1;
    else if (at->tv_usec < bt->tv_usec)
        return -1;
    else if (at->tv_usec > bt->tv_usec)
        return 1;
    else
        return 0;
}

void db_open(char *name, int join)
{
    int ret;
    u_int32_t flags, flags1, flags0;

    flags0 = DB_CREATE|DB_TEMPORAL;
    flags1 = DB_TEMPORAL|DB_AUDIT;

    if (join == 1)
        flags = flags1;
    else
        flags = flags0;
    if ((ret = db_create(&db, dbenv, 0)) != 0) {
        dbenv->err(dbenv, ret, "db_create");
        exit (1);
    }

/*
    if ((ret = db->set_bt_compare(db, key_comp)) != 0) {
        dbenv->err(dbenv, ret, "DB->set_bt_compare");
        exit (1);
    }
*/

    if ((ret = db->open(db, name, NULL,
                        DB_BTREE, flags, 0664)) != 0) {
        dbenv->err(dbenv, ret, "db->open: %s", name);
        exit (1);
    }
}


/* Added by Huilong, Modified by Man Zhang 04-20-2007
 */
void SendEmail(SQLHDBC dbc)
{
    char command[256];

    char msgFileName[FILENAMELEN]="warning";
    FILE *msgFile;

    char CSIemail[EMAILLEN];
    char CSOemail[EMAILLEN];
    char DBAemail[EMAILLEN];


    char createID[IDLEN];

    getCreateID(dbc,createID);

    strcat(msgFileName,createID);

    //Send email
    msgFile=fopen(msgFileName,"w");
    fprintf(msgFile,"Database tampered at: \nDbPath---%s\nDbName---%s\n",ConfigTable.Dbpath,ConfigTable.Dbname);

    printf("Send email with content [Database tampered at: \nDbPath---%s\nDbName---%s\n]!\n", ConfigTable.Dbpath,ConfigTable.Dbname);

    if (fclose(msgFile)) {
 	show_val_error();
    }


    if (!getEmailAddr(dbc,CSIemail,"C")) {// CSI
	insErrEvent(dbc,"Fail to read CSI from IsMaintainedBy table!");
	fprintf(stderr,"Fail to read from IsMaintainedBy table!\n");
    }

    if (!getEmailAddr(dbc,CSOemail,"A")) {//CSO
	insErrEvent(dbc,"Fail to read CSO from IsMaintainedBy table!");
	fprintf(stderr,"Fail to read from IsMaintainedBy table!\n");
    }

    if (!getEmailAddr(dbc,DBAemail,"D")) { //DBA
	insErrEvent(dbc,"Fail to read DBA from IsMaintainedBy table!");
	fprintf(stderr,"Fail to read from IsMaintainedBy table!\n");
    }


    command[0]='\0';

    //Modified by Man,2007,0610
    //sprintf(command, "mail %s -c %s -c %s -s Warning!<  warning.msg", CSIemail,CSOemail,DBAemail);
    sprintf(command, "mail %s -c %s -c %s -s Warning!<  %s", CSIemail,CSOemail,DBAemail,msgFileName);
    //end

    printf("Send email by command[%s]!\n", command);
    
    system(command);

    command[0]='\0';
    sprintf(command,"rm -f %s",msgFileName);
    printf("delete file by command:\n",command);

    // delete not Res File
    if (system(command) == -1) {
        insErrEvent(dbc,"Can't delete notResFile");
    }


}


/* end Huilong
 */

// implement forensic analysis when tampering happens
// Modified By Man, 06/03/2007
//void MonoForensic(SQLHDBC dbc, int fail_flag, char *server, in_port_t port, char *maxPreNotID)
void MonoForensic(SQLHDBC dbc, VerificationParameter* ver_param,
                  in_port_t port, char *maxPreNotID) {
    //SQLHSTMT stmt;
    //SQLSMALLINT columns; /* number of columns in result-set */
    //int row = 0;
    //SQLINTEGER indicator;
    //char buf[512];
//    SQLUSMALLINT i=1;
    //SQLCHAR getNot[] = "select ActualDate,ActualTime,ActualMilliseconds,EventID,EventDate,EventTime from Event where Kind='N' AND Dbpath='";

    //SQLRETURN sqlret; /* ODBC API return status */

    AUD_TXN_HASH_DATA txn_hash_data;
    char hash_value[AUD_HASH_LEN];

    char stopactualdate[DATELEN],stopactualtime[DATELEN], stopactualms[DATELEN];
    struct timeval ttime;

    NID oldnid;

    VER_TOL_ENTRY *p;
    //char digest[AUD_HASH_LEN];
    TTDATA *ttdata;

    rb_red_blk_node *np;

    int sockfd;
    struct hostent *serv_p;
    SAI servaddr;

    int n;

    SQLCHAR getFstNotEvt[SQLQUERY] = "select ActualDate,ActualMilliseconds,EventID from Event where EventID=( select MIN(EventID) from Event where Kind='N' AND Path='";

    // get next Notarization Event
    SQLCHAR getNextNotEvt[SQLQUERY];
//    SQLCHAR getNextNotEvt[SQLQUERY] = "select ActualDate,ActualMilliseconds,EventID from Event where EventID=(select MIN(EventID) from Event where Kind='N' AND Path='";

    strcat(getFstNotEvt,ConfigTable.Dbpath);
    strcat(getFstNotEvt,"' AND Name='");
    strcat(getFstNotEvt,ConfigTable.Dbname);
    strcat(getFstNotEvt,"')");


    char notActDate[TIMESTPLEN],notActMs[MSLEN],notEventID[IDLEN];

printf("\nGet First Notarization Event:[%s]\n",getFstNotEvt);

    if (!getNotEvent(dbc,getFstNotEvt,notActDate,notActMs,notEventID)) {
        insErrEvent(dbc,"No Notarization Event found when running forensic analysis");
        fprintf(stderr,"No Notarization Event found when running forensic analysis\n");
        exit(1);
    }

    printf("\nBegin to do forensic analysis\n");
//    print_tol();

    np = RBMin(tree);
    p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
    memset(hash_value, 0, AUD_HASH_LEN);

    char ChainID[IDLEN];
    int MoreNotCheck = 1;
    char curTamperID[IDLEN];
    char Result[IDLEN];

    // get current Tampering ID
    getTamperID(dbc,curTamperID);

    while (1)
    {

        /* Rechain the txn's in TOL till the tail time "ttime"
        * and put the tail hash_value in "hash_value" for verification.
        */
        /* If tree is null,no hash value recalculated */
        if (p == NULL) {
            break;
        }


        /* get each tuple corresponding every Notarizer event for this DB
        * Recalculate each hash_value and NID for every Notarizer Interval
        */

        // set the stop time for this Notarize Event
        getDate(notActDate,stopactualdate,stopactualtime);
        ttime = mkCalTime(stopactualdate,stopactualtime,notActMs);
printf("\nRecalculate hash value until time(%d,%d)\n",ttime.tv_sec,ttime.tv_usec);

        // recalculate hash value for each Notarize Event

        while (p != NULL) {
            if (time_cmp(p->time, ttime) >= 0)
                break;

//            printf("TOL:txn_time(%d,%d)\n",
//                   p->time.tv_sec, p->time.tv_usec);
	    printf("    start hash_value = ");
            print_hash(hash_value);

            printf("    txn_hash =");
            print_hash(p->hash_value);

            memcpy(txn_hash_data.prev_hash_value, hash_value, AUD_HASH_LEN);
            txn_hash_data.time = p->time;
            memcpy(txn_hash_data.hash_value, p->hash_value, AUD_HASH_LEN);
            if (__aud_hash(&txn_hash_data, sizeof(AUD_TXN_HASH_DATA), hash_value) != 0) { 
		insErrEvent(dbc,"Error in call __aud_hash API!");
                fprintf(stderr,"Error in call __aud_hash API!");
                exit(1);
            }

            printf("    tail_hash=");
            print_hash(hash_value);

   //     if (time_cmp(p->time, ttime) == 0)

            np = TreeSuccessor(tree, np);

	    if (tree->nil == np ) {
                break;
            }

            p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);

        }

        // send hash value to surety to get NotarID
        /* Establish UDP */

	//hash the hash_value and store it as nid for the future purpose of forensic analysis
	sha1Param param;

	/* Do the hashing */
	if (sha1Reset(&param))
	    exit(1);
	if (sha1Update(&param, hash_value, AUD_HASH_LEN))
	    exit(1);
	if (sha1Digest(&param, ver_param->nid_output.nid))
	    exit(1);
	gettimeofday(&(ver_param->nid_output.ntime), NULL);


fprintf(stderr, "executing mono!\n");

    ver_param->hash_value = (char*) malloc (AUD_HASH_LEN);
    memcpy(ver_param->hash_value, hash_value, AUD_HASH_LEN);
    memcpy(&(ver_param->ttime), &ttime, sizeof(ttime));
    CommunicateWithDNS(ver_param);
    free(ver_param->hash_value);

        // get original NID stored in Chain table
        getNIDfromChain(dbc,oldnid.nid,notEventID);

printf("\ncurrent TamperID:%s\n",curTamperID);

	strcpy(Result,"P");

        // compare two NotarID to detect corruption
        if (memcmp(oldnid.nid, ver_param->nid_output.nid, AUD_HASH_LEN) != 0) {
	    strcpy(Result,"F");
            printf("Find corruption HERE!\n");
            printf("    NID stored in MySql:\n");
            print_hash(oldnid.nid);
            printf("    Data recomputed from the database:\n");
            print_hash(ver_param->nid_output.nid);

            // store the corruption region in Failure table
            UpdateRegion(dbc,notEventID,maxPreNotID,curTamperID);


	    // restore the hashvalue to the noncorruption in the mySql
printf("\noriginal hash value:\n");
print_hash(hash_value);
	    getHashvalueFromChain(dbc,hash_value,notEventID);
printf("\noNew hash value:\n");
print_hash(hash_value);
 	    //memcpy(hash_value, rnid.nid, AUD_HASH_LEN);
        }


        getChainID(dbc,notEventID,ChainID);

        insNotCheck(dbc,ChainID,Result,curTamperID);

        // get next Notarization Event
	bzero(getNextNotEvt,SQLQUERY);

	strcpy(getNextNotEvt,"select ActualDate,ActualMilliseconds,EventID from Event where EventID=(select MIN(EventID) from Event where Kind='N' AND Path='");

        strcat(getNextNotEvt,ConfigTable.Dbpath);
        strcat(getNextNotEvt,"' AND Name='");
        strcat(getNextNotEvt,ConfigTable.Dbname);
        strcat(getNextNotEvt,"' AND EventID>");
        strcat(getNextNotEvt,notEventID);
        strcat(getNextNotEvt," AND EventID<=");
        strcat(getNextNotEvt,maxPreNotID);
        strcat(getNextNotEvt,")");

printf("\nGet Next Notarization Event:[%s]\n",getNextNotEvt);

        if (!getNotEvent(dbc,getNextNotEvt,notActDate,notActMs,notEventID)) {
	    MoreNotCheck = 0;
	    break;
        }

/*            np = TreeSuccessor(tree, np);

            if (tree->nil == np ) {
                break;
            }

            p = (np == NULL)? NULL: (VER_TOL_ENTRY *)(np->info);
*/
    }

    if (MoreNotCheck) {
        // add more 'P' tuples for the rest of Forensic analysis in NotarizationCheck

        strcpy(Result,"P");
	while (1) {
            getChainID(dbc,notEventID,ChainID);

            insNotCheck(dbc,ChainID,Result,curTamperID);

	    bzero(getNextNotEvt,SQLQUERY);

	    strcpy(getNextNotEvt,"select ActualDate,ActualMilliseconds,EventID from Event where EventID=(select MIN(EventID) from Event where Kind='N' AND Path='");

            strcat(getNextNotEvt,ConfigTable.Dbpath);
            strcat(getNextNotEvt,"' AND Name='");
            strcat(getNextNotEvt,ConfigTable.Dbname);
            strcat(getNextNotEvt,"' AND EventID>");
            strcat(getNextNotEvt,notEventID);
            strcat(getNextNotEvt," AND EventID<=");
            strcat(getNextNotEvt,maxPreNotID);
            strcat(getNextNotEvt,")");

printf("\nGet Next Notarization Event:[%s]\n",getNextNotEvt);

            if (!getNotEvent(dbc,getNextNotEvt,notActDate,notActMs,notEventID)) {
	        break;
            }
	}

/* Added by Huilong to break the loop */
           // break;  // Modified by Man,06/04/2005
/* End Huilong */
    }    

}


void UpdateRegion(SQLHDBC dbc, char *failNotID, char *maxNotID, char *curTamperID)
{
    char preNotID[IDLEN],preMaxNotID[IDLEN],tmpNotID[IDLEN];

printf("\n Begin to update Region Table now.\n");

    getPreNotID(dbc,preNotID,failNotID);
printf("\nprevious corruption NotID:%s\n",preNotID);

    int i = 0;
    bzero(tmpNotID,IDLEN);
    strcpy(tmpNotID,maxNotID);

    while (i < ConfigTable.VFactor) {
        getPreNotID(dbc,preMaxNotID,tmpNotID);

    	bzero(tmpNotID,IDLEN);
    	strcpy(tmpNotID,preMaxNotID);

	i++;
    }
printf("\nprevious most recent Notarization NotID:%s\n",preMaxNotID);
//    getTamperID(dbc,curTamperID);
//printf("\ncurrent TamperID:%s\n",curTamperID);

/* Changed by Huilong for bug */
//    if (strcpy(preNotID,preMaxNotID) == 0) {
   if (strcmp(preNotID,preMaxNotID) == 0) {
/* End Huilong */
	//triangle
    	WriteSynRegion(dbc,curTamperID,preNotID,maxNotID,failNotID,maxNotID,
                preNotID,preMaxNotID,failNotID,failNotID);

    } else {
	// rectangular
        WriteSynRegion(dbc,curTamperID,preNotID,maxNotID,failNotID,maxNotID,
                preNotID,preMaxNotID,failNotID,preMaxNotID);
    }
}


// insert into SynchoronizedRegion
void WriteSynRegion(SQLHDBC dbc, char *FailureID,
                 char *TopLeftX, char *TopLeftY,
                 char *TopRightX, char *TopRightY,
                 char *BottomLeftX, char *BottomLeftY,
                 char *BottomRightX, char *BottomRightY
                 )
{

printf("\nWrite Region(topleftx,toplefty,toprightx,toprighty,bottomleftx,bottomlefty,bottomrightx,bottomrighy:[%s],[%s]\n[%s],[%s]\n[%s],[%s]\n[%s],[%s]\n",TopLeftX,TopLeftY,TopRightX,TopRightY,BottomLeftX,BottomLeftY,BottomRightX,BottomRightY);

    SQLCHAR insRegion[MAXINSLEN] = "insert into SynchronizedRegion (EventID, TopLeftX, TopLeftY, TopRightX, TopRightY, BottomLeftX, BottomLeftY, BottomRightX, BottomRightY) VALUES (";

    strcat(insRegion, FailureID);
    strcat(insRegion,",");
    strcat(insRegion, TopLeftX);
    strcat(insRegion,",");
    strcat(insRegion, TopLeftY);
    strcat(insRegion,",");
    strcat(insRegion, TopRightX);
    strcat(insRegion,",");
    strcat(insRegion, TopRightY);
    strcat(insRegion,",");
    strcat(insRegion, BottomLeftX);
    strcat(insRegion,",");
    strcat(insRegion, BottomLeftY);
    strcat(insRegion,",");
    strcat(insRegion, BottomRightX);
    strcat(insRegion,",");
    strcat(insRegion, BottomRightY);
    strcat(insRegion,")");

printf("\ninsRegion:[%s]\n",insRegion);
    exeSqlQuery(dbc,insRegion);

    printf("Region stored in the Region table.\n");
}
